      program cjdet
      include 'implno.dek'
      include 'const.dek'
      include 'vector_eos.dek'
      include 'network.dek'
      include 'cjdet.dek'


! evaluates a chapman-jouget detonation

! declare
      character*80   :: string
      integer        :: i,k,kkase
      real*8         :: xh1,xhe4,xc12,xo16,xne22,qdum,mach_fac


! formats
01    format(1x,a)
03    format(a)


! initialize the network
      call net_initialize
      call zet127
      call init_torch

! read the helmeos table
      call read_helm_table



! get the upstream consitions
      write(6,01) 'upstream temp, den, h1, he4, c12, o16 ne22 =>'
      read (5,*)  temp_up,den_up,xh1,xhe4,xc12,xo16,xne22


! see if we want a driven detonation
      write(6,*) ' '
      write(6,*) 'to drive a detonation give the mach number,'
      write(6,*) 'otherwise <cr> for an undriven detonation =>'
      string = ' ' 
      read(5,03) string
      if (string(1:1) .ne. ' ') then
       read(string,*) mach_fac
      else
       mach_fac = 0.0d0
      end if


! set the upsteam composition variables
      xmass_up(1:ionmax) = 1.0d-30
      if (ih1  .ne. 0)  xmass_up(ih1)  = xh1
      if (ihe4 .ne. 0)  xmass_up(ihe4) = xhe4
      if (ic12 .ne. 0)  xmass_up(ic12) = xc12
      if (io16 .ne. 0)  xmass_up(io16) = xo16
      if (ine22 .ne. 0) xmass_up(ine22) = xne22
      xmass_up(1:ionmax) = min(1.0d0, max(xmass_up(1:ionmax),1.0d-30) )


! get the chapman-jouget detonation solution
      kkase = 1
      mach  = 0.0d0

      call cjsolve(kkase,xmass_up,temp_up,den_up,mach, &
                   qburn_cj,xmass_cj,ener_up,pres_up,cs_up, &
                   vel_det,vel_cj,temp_cj,den_cj,ener_cj,pres_cj,cs_cj)


! get shock solution
      kkase = 4
      mach_sh = vel_det/cs_up

      call cjsolve(kkase,xmass_up,temp_up,den_up,mach_sh, &
                   qdum,xmass_up,ener_up,pres_up,cs_up, &
                   vel_det,vel_sh,temp_sh,den_sh,ener_sh,pres_sh,cs_sh)


! for driven detonations

      if (mach_fac .ne. 0.0) then
       mach = mach_fac * mach_sh


! get the shock values for this mach number
       kkase = 4
       call cjsolve(kkase,xmass_up,temp_up,den_up,mach, &
                    qdum,xmass_up,ener_up,pres_up,cs_up, &
                    vel_det,vel_swsh,temp_swsh,den_swsh, &
                    ener_swsh,pres_swsh,cs_swsh)


! get the strong shock point
       kkase = 2
       call cjsolve(kkase,xmass_up,temp_up,den_up,mach, &
                    qburn_s,xmass_s,ener_up,pres_up,cs_up, &
                    vel_det,vel_s,temp_s,den_s,ener_s,pres_s,cs_s)

! get the weak shock point
       kkase = 3
       call cjsolve(kkase,xmass_up,temp_up,den_up,mach, &
                    qburn_w,xmass_w,ener_up,pres_up,cs_up, &
                    vel_det,vel_w,temp_w,den_w,ener_w,pres_w,cs_w)
      end if



! write the out the results
      call cjout(6)

      end program cjdet







      subroutine cjout(lu)
      include 'implno.dek'
      include 'vector_eos.dek'
      include 'network.dek'
      include 'cjdet.dek'

! writes the output to logical unit lu

! declare the pass
      integer    :: lu

! local variables
      integer    :: i,indx(ionmax)
      real*8     :: xsum,dum,tau_qse,tau_nse,tau_burn,abar,zbar

! formats
01    format(1x,a7,'=',1pe10.3,' ',a7,'=',1pe10.3,' ', a7,'=',1pe10.3)
02    format(1x,a4,'=',1pe10.3,' ',a4,'=',1pe10.3,' ', a4,'=',1pe10.3,' ',a4,'=',1pe10.3)
03    format(1x,a)



      write(lu,*)  ' '
      write(lu,03) 'upstream state:'
      write(lu,01) 'temp_up',temp_up,'den_up ',den_up, 'pres_up',pres_up
      write(lu,01) 'cs_up  ',cs_up


      write(lu,*)  ' '
      write(lu,03) 'state behind shock:'
      write(lu,01) 'temp_sh',temp_sh,'den_sh ',den_sh, 'pres_sh',pres_sh
      write(lu,01) 'cs_sh  ',cs_sh, 'vel_mat',vel_sh,'vel_det',vel_det
      write(lu,01) 'mach_sh',mach_sh


      write(lu,*)  ' '
      write(lu,03) 'cj state (should be sonic with vel_mat = cs_cj):'
      write(lu,01) 'temp_cj',temp_cj,'den_cj ',den_cj, 'pres_cj',pres_cj
      write(lu,01) 'cs_cj  ',cs_cj, 'vel_mat',vel_cj,'vel_det',vel_det
      write(lu,01) 'mach_cj',vel_cj/cs_cj,'qburn_cj',qburn_cj



! nse composition at the cj point
      xsum = sum(xmass_cj(1:ionmax))
      xsum = 1.0d0 - xsum

      write(lu,*) ' '
      write(lu,*) 'top 10 cj nse mass fractions:'
      call indexx(ionmax,xmass_cj,indx)
      write(lu,02) (ionam(indx(i)), xmass_cj(indx(i)), i=ionmax,ionmax-9,-1), '1-sum',xsum



! qse and nse time and length scales, calder et al 2007
      tau_qse = exp(149.7d0/(temp_cj*1.0d-9) - 37.7d0)
      tau_nse = den_cj**(0.2d0)*exp(179.7d0/(temp_cj*1.0d-9) - 39.0d0)

      write(lu,*) ' '
      write(lu,*) 'cj detonation time and length scales'
      write(lu,01) 'tau_qse',tau_qse,'len_qse',tau_qse*vel_det
      write(lu,01) 'tau_nse',tau_nse,'len_nse',tau_nse*vel_det
      write(lu,*) ' '



! the weak and strong detonations
      if (qburn_w .ne. 0.0) then
       write(lu,*)  ' '
       write(lu,*) 'strong and weak detonation shock state:'
       write(lu,01) 'temp_sw=',temp_swsh,' den_sw=',den_swsh,' pres_sw=',pres_swsh
       write(lu,01) 'cs_sw  =',cs_swsh,' vel_sw=',vel_swsh,' vel_det=',vel_det
       write(lu,01) 'mach_sw=',mach



! strong detonations
       dum = 0.0d0
       if (cs_s .ne. 0.0) dum = vel_s/cs_s

       write(lu,*)  ' '
       write(lu,*) 'strong detonation (subsonic with mach_s < 1):'
       write(lu,01) 'temp_s =',temp_s, ' den_s =',den_s,' pres_s =',pres_s
       write(lu,01) 'cs_s   =',cs_s, ' vel_s =',vel_s,' vel_det=',vel_det
       write(lu,01) 'mach_s =',dum,' qburn_s =',qburn_s

       xsum = sum(xmass_s(1:ionmax))
       xsum = 1.0d0 - xsum

       write(lu,*) ' '
       write(lu,*) 'top 10 strong detonation nse mass fractions:'
       call indexx(ionmax,xmass_s,indx)
       write(lu,02) (ionam(indx(i)), xmass_s(indx(i)), i=ionmax,ionmax-9,-1), '1-sum',xsum
       write(lu,*) ' '



! weak detonations
       dum = 0.0d0
       if (cs_w .ne. 0.0) dum = vel_w/cs_w

       write(lu,*)  ' '
       write(lu,*) 'weak detonation (supersonic mach_w > 1):'
       write(lu,01) 'temp_w =',temp_w,' den_w =',den_w, ' pres_w =',pres_w
       write(lu,01) 'cs_w   =',cs_w, ' vel_w =',vel_w,' vel_det=',vel_det
       write(lu,01) 'mach_w =',dum,' qburn_w =',qburn_w

       xsum = sum(xmass_s(1:ionmax))
       xsum = 1.0d0 - xsum

       write(lu,*) ' '
       write(lu,*) 'top 10 weak detonation nse mass fractions:'
       call indexx(ionmax,xmass_w,indx)
       write(lu,02) (ionam(indx(i)), xmass_w(indx(i)), i=ionmax,ionmax-9,-1), '1-sum',xsum
       write(lu,*) ' '


      end if

      return
      end subroutine cjout



      include 'weaktab.f90'
      include 'net_initialize.f90'
      include 'net_auxillary.f90'
      include 'net_torch.f90'
      include 'nsesub.f90'
      include 'azbar.f90'
      include 'helmholtz_for_public.f90'
      include 'support.f90'
      include 'glue_for_public.f90'
      include 'cjsub.f90'
