! this routine contains auxillary network routine


! routine read_partition_functions does what it says on the rauscher & thielemann data
! routine get_partition_functions interpolates the rauscher & thielemann partition functions
! routine init_isotope_rate_pointers initializes integer name & reaction rate pointers for the networks




      subroutine read_partition_functions
      include 'implno.dek'
      include 'const.dek'
      include 'network.dek'
!
! this routine reads and initializes the rauscher & thielemann partition function data
!
! declare

      character*132    :: string
      integer          :: i,j,izpf,iapf,inpf
      double precision :: xx


! initialize
      pf(1:pf_tmax,1:pf_zmax,1:pf_nmax) = -101.0d0 
      pf_gspin(1:pf_zmax,1:pf_nmax)     = -101.0d0

! open the partition function data file and read the header
      open(unit=22, file='partition_function_frdm.dat', status='old')
      do i=1,4
       read(22,11) string
11     format(a)
      enddo

! read the file
      do i=1,40000
       read(22,11,end=30) string
       if (string(1:12) .eq. 'END OF TABLE') goto 30
       read(22,*) izpf,iapf,xx
       inpf = iapf - izpf
       if (izpf .lt. 1) stop 'izpf < 1 during read of partition functions'
       if (inpf .lt. 1) stop 'inpf < 1 during read of partition functions'
       if (izpf .gt. pf_zmax) stop 'izpf > pf_izmax during read of partition functions'
       if (inpf .gt. pf_nmax) stop 'inpf > pf_inmax during read of partition functions'
       pf_gspin(izpf,inpf) = xx
       read(22,*) (pf(j,izpf,inpf), j=1,24)
      end do

! close up
30    close(unit=22)


! set the temperatures on which the partition functions are calculated
      pf_t9 = (/0.01d0, 0.15d0,  0.2d0, 0.3d0, 0.4d0, 0.5d0, 0.6d0, 0.7d0, &
                 0.8d0,  0.9d0,  1.0d0, 1.5d0, 2.0d0, 2.5d0, 3.0d0, 3.5d0, & 
                 4.0d0,  4.5d0,  5.0d0, 6.0d0, 7.0d0, 8.0d0, 9.0d0, 10.0d0/)

      return
      end




      subroutine get_partition_functions(t9)
      include 'implno.dek'
      include 'burn_common.dek'
      include 'vector_eos.dek'
      include 'network.dek'


! this routine generates rauscher & thielemann partition function values

! declare the pass
     double precision :: t9


! declare locals
! for the partition function interpolation
      integer            :: i,jat
      integer, parameter :: norder=2
      double precision   :: x,x1,x2,a,b,e,alfa,beta,stat_weight


! waprt(i) is the temperature dependent partition functions
! zwork1(i) is the ratio of wpart(i) to the ground state partition function
! zwork2(i) is the temperature derivative of zwork1
! initialize


      zwork1(ionbeg:ionend) = 1.0d0
      zwork2(ionbeg:ionend) = 0.0d0
      wpart(ionbeg:ionend)  = 1.0d0

      if (ineut .ne. 0) then
       zwork1(ineut) = 1.0d0
       zwork2(ineut) = 0.0d0
       wpart(ineut)  = 2.0d0
      end if

      if (iprot .ne. 0) then
       zwork1(iprot) = 1.0d0
       zwork2(iprot) = 0.0d0
       wpart(iprot)  = 2.0d0
      end if

      if (ih1 .ne. 0) then
       zwork1(ih1) = 1.0d0
       zwork2(ih1) = 0.0d0
       wpart(ih1)  = 2.0d0
      end if

      if (ihe4 .ne. 0) then
       zwork1(ihe4) = 1.0d0
       zwork2(ihe4) = 0.0d0
       wpart(ihe4)  = 1.0d0
      end if



! for every isotope not named n, p or alpha

      do i=ionbeg,ionend

! bullet check
       if (int(zion(i)) .lt. 1) stop 'izpf < 1 during read of partition functions'
       if (int(nion(i)) .lt. 1) stop 'izpf < 1 during read of partition functions'
       if (int(zion(i)) .gt. pf_zmax) stop 'izpf > pf_izmax during read of partition functions'
       if (int(nion(i)) .gt. pf_nmax) stop 'inpf > pf_inmax during read of partition functions'


! if isotope is in the network 
       if (pf_gspin(int(zion(i)),int(nion(i))) .lt. 0.0d0) cycle

!       write(6,*) int(zion(i)),int(nion(i)),pf_gspin(int(zion(i)),int(nion(i)))


        stat_weight = 2.0d0 * pf_gspin(int(zion(i)),int(nion(i))) + 1.0d0

! clamp at both ends of the temperature grid
       if (t9 .le. pf_t9(1)) then
        zwork1(i) = pf(1,int(zion(i)),int(nion(i)))
        zwork2(i) = 0.0d0
        wpart(i)  = zwork1(i) * stat_weight

       else if (t9 .ge. pf_t9(pf_tmax)) then
        zwork1(i) = pf(pf_tmax,int(zion(i)),int(nion(i)))
        zwork2(i) = 0.0d0
        wpart(i)  = zwork1(i) * stat_weight

! linear larange interpolation

       else
        call locate(pf_t9,pf_tmax,t9,jat)
        jat = max(1,min(jat - norder/2 + 1,pf_tmax - norder + 1))

!        write(6,*) jat,t9
!        write(6,*) pf_t9(jat),pf_t9(jat+1)
!        write(6,*) pf(jat,int(zion(i)),int(nion(i))),pf(jat+1,int(zion(i)),int(nion(i)))

        x  = t9
        x1 = pf_t9(jat)
        x2 = pf_t9(jat+1)
        a  = x - x1
        b  = x - x2
        e  = x1 - x2
        alfa =  b/e
        beta = -a/e
        zwork1(i) = alfa * pf(jat,int(zion(i)),int(nion(i))) + beta * pf(jat+1,int(zion(i)),int(nion(i)))
        zwork2(i) = 0.0d0
        wpart(i)  = zwork1(i) * stat_weight

! no partition functions
!       zwork1(i) = 1.0d0
!       zwork2(i) = 0.0d0
!       wpart(i)  = 1.0d0



       end if

!       write(6,156) i, ionam(i), zwork1(i), wpart(i)
!156    format(1x,i4,' ',a,' ',1p2e12.4)
!       read(5,*)

      enddo
      return
      end








      subroutine init_isotope_rate_pointers
      include 'implno.dek'
      include 'network.dek'

! initializes all the integer pointers for the isotopes and rates


! zero all the isotope pointers
      ih1=0   ; iprot=0 ; ineut=0 ; ihe4=0  ; ih2=0   ; ih3=0   ; ihe3=0  ; ili6=0  ; ili7=0  ; ili8=0
      ibe7=0  ; ibe9=0  ; ib8=0   ; ib9=0   ; ib10=0  ; ib11=0  ; ic11=0  ; ic12=0  ; ic13=0  ; ic14=0
      in12=0  ; in13=0  ; in14=0  ; in15=0  ; io14=0  ; io15=0  ; io16=0  ; io17=0  ; io18=0  ; if17=0
      if18=0  ; if19=0  ; ine18=0 ; ine19=0 ; ine20=0 ; ine21=0 ; ine22=0 ; ine23=0 ; ina20=0 ; ina21=0
      ina22=0 ; ina23=0 ; img22=0 ; img23=0 ; img24=0 ; img25=0 ; img26=0 ; ial25=0 ; ial26=0 ; ial27=0
      ial28=0 ; isi26=0 ; isi27=0 ; isi28=0 ; isi29=0 ; isi30=0 ; ip28=0  ; ip29=0  ; ip30=0  ; ip31=0
      is30=0  ; is31=0  ; is32=0  ; is33=0  ; is34=0  ; icl35=0 ; icl37=0 ; iar36=0 ; iar37=0 ; iar38=0
      ik39=0  ; ica40=0 ; isc43=0 ; isc45=0 ; iti44=0 ; iti46=0 ; iti48=0 ; iti50=0 ; iv46=0  ; iv47=0
      iv48=0  ; iv51=0  ; icr47=0 ; icr48=0 ; icr49=0 ; icr50=0 ; icr51=0 ; icr52=0 ; icr53=0 ; icr54=0
      icr56=0 ; imn50=0 ; imn51=0 ; imn52=0 ; imn55=0 ; ife52=0 ; ife54=0 ; ife55=0 ; ife56=0 ; ife57=0
      ife58=0 ; ife59=0 ; ico54=0 ; ico55=0 ; ico56=0 ; ico59=0 ; ini56=0 ; ini57=0 ; ini58=0 ; ini59=0
      ini64=0 ; ini66=0 ; icu63=0 ; izn60=0 ; izn64=0


! zero all the rate pointers

      ir3a=0   ;  irg3a=0  ;  ircag=0  ;  ir1212=0 ;  ir1216=0 ;  iroga=0 
      iroag=0  ;  irnega=0 ;  irneag=0 ;  irmgga=0 ;  irmgag=0 ;  irsiga=0   
      irmgap=0 ;  iralpa=0 ;  iralpg=0 ;  irsigp=0 ;  irsiag=0 ;  irsga=0    
      irsiap=0 ;  irppa=0  ;  irppg=0  ;  irsgp=0  ;  irsag=0  ;  irarga=0   
      irsap=0  ;  irclpa=0 ;  irclpg=0 ;  irargp=0 ;  irarag=0 ;  ircaga=0   
      irarap=0 ;  irkpa=0  ;  irkpg=0  ;  ircagp=0 ;  ircaag=0 ;  irtiga=0   
      ircaap=0 ;  irscpa=0 ;  irscpg=0 ;  irtigp=0 ;  irtiag=0 ;  ircrga=0   
      irtiap=0 ;  irvpa=0  ;  irvpg=0  ;  ircrgp=0 ;  ircrag=0 ;  irfega=0   
      ircrap=0 ;  irmnpa=0 ;  irmnpg=0 ;  irfegp=0 ;  irfeag=0 ;  irniga=0   
      ir1616=0 ;  iraan=0

      irpp=0   ;  ir33=0   ;  ircpg=0  ;  irnpg=0  ;  iropg=0   ;  irnag=0 
      irfeap=0 ;  ircopa=0 ;  ircopg=0 ;  irnigp=0 ;  irfepg=0  ;  ircogp=0 
      ir52ng=0 ;  ir53gn=0 ;  ir53ng=0 ;  ir54gn=0 ;  irheng=0  ;  irhegn=0   
      irhng=0  ;  irhegp=0 ;  irpen=0  ;  irnep=0  ;  irn56ec=0 ;  ifa=0      
      ifg=0    

      ir54ng=0      ; ir55gn=0       ; ir55ng=0      ; ir56gn=0    
      irfe54ap=0    ; irco57pa=0     ; irfe56pg=0    ; irco57gp=0 
      irfe56_aux1=0 ;  irfe56_aux2=0 ; irfe56_aux3=0 ; irfe56_aux4=0

      irbepg=0 ;  irb8ep=0 ;  irbeec=0 ;  irpep=0 ;  irhep=0

      irc12pg=0  ;  irn13gp=0  ;  irn13enu=0  ;  irc13pg=0   ; irn14gp=0 
      irn14pg=0  ;  iro15gp=0  ;  iro15enu=0  ;  irn15pa=0   ; irc12ap=0    
      irn15pg=0  ;  iro16gp=0  ;  iro16pg=0   ;  irf17gp=0   ; irf17enu=0   
      iro17pa=0  ;  irn14ap=0  ;  iro17pg=0   ;  irf18gp=0   ; irf18enu=0   
      iro18pa=0  ;  irn15ap=0  ;  iro18pg=0   ;  irf19gp=0   ; irf19pa=0    
      iro16ap=0  ;  irn13pg=0  ;  iro14gp=0   ;  iro14enu=0  ; iro14ap=0    
      irf17pa=0  ;  irf17pg=0  ;  irne18gp=0  ;  irne18enu=0 ; irf18pa=0    
      iro15ap=0  ;  irne18ap=0 ;  iro15ag=0   ;  irne19ga=0  ; irne19pg=0   
      irna20gp=0 ;  irsi26ap=0 ;  irne19enu=0 ;  irne20pg=0  ; irna21gp=0

      irweakn=0 ;     irweakp=0 ;     irpng=0 ;       irdgn=0 ;      irdpn=0 
      ir2pnp=0 ;      irdng=0 ;       irtgn=0 ;       irddp=0 ;      irtpd=0       
      irdpg=0 ;       irhe3gp=0 ;     irddn=0 ;       irhe3nd=0 ;    irtpn=0       
      irhe3np=0 ;     irddg=0 ;       irhe4gd=0 ;     irtpg=0 ;      irhe4gp=0 
      irtdn=0 ;       irhe4nd=0 ;     irtt2n=0 ;      irhe42nt=0 ;   irhe3ng=0     
      irhe4gn=0 ;     irhe3dp=0 ;     irhe4pd=0 ;     irhe3td=0 ;    irhe4dt=0     
      irhe3tnp=0 ;    irhe4pnt=0 ;    ir33inv=0 ;     irhe4npg=0 ;   irli6gpn=0    
      irdag=0 ;       irli6ga=0 ;     irtan=0 ;       irli6na=0 ;    irli6pa=0     
      irhe3ap=0 ;     irtag=0 ;       irli7ga=0 ;     irli7pa=0 ;    ir2he4ga=0 
      irli7dn=0 ;     ir2and=0 ;      irli7t2n=0 ;    ir2a2nt=0 ;    irli7he3np=0 
      ir2apnhe3=0 ;   irhe3ag=0 ;     irbe7ga=0 ;     irli6pg=0 ;    irbe7gp=0   
      irli7pn=0 ;     irbe7np=0 ;     irbe7dp=0 ;     ir2apd=0 ;     irbe7tnp=0    
      ir2apnt=0 ;     irbe7he32p=0 ;  ir2a2phe3=0 ;   irbe9pa=0 ;    irli6ap=0     
      irbe7pg=0 ;     irb8gp=0 ;      irli6ag=0 ;     irb10ga=0 ;    irli7an=0     
      irb10na=0 ;     irbe9pg=0 ;     irb10gp=0 ;     irb10pa=0 ;    irbe7ap=0     
      irli7ag=0 ;     irb11ga=0 ;     irb11pa=0 ;     ir2he4ap=0 ;   irbe7ag=0     
      irc11ga=0 ;     irb11pn=0 ;     irc11np=0 ;     irb8ap=0 ;     irc11pa=0      
      irb10pg=0 ;     irc11gp=0 ;     irc11na=0 ;     ir3an=0 ;      irbe9an=0     
      irc12na=0 ;     irb11pg=0 ;     irc12gp=0 ;     irb11ap=0 ;    irc14pa=0     
      irli6ng=0 ;     irli7gn=0


      irh3em=0 ;    irc11em=0 ;   iraanbe7=0 ;  irdap=0 ;     irtap=0 ;   irb10ng=0 
      irbe9dn=0 ;   irb10dp=0 ;   irb8npa=0 ;   irbe9pda=0 ;  ir2adp=0 ;  irb11p2a=0 
      irc11n2a=0


      irh3pg=0 ;    irli7pag=0 ;  irgaan=0 ;    irbe9pd=0 ;   ir3ap=0 ;    ir1212p=0 ;   irna23pc=0 
      ir1212n=0 ;   irmg23nc=0 ;  ir1212a=0 ;   irne20ac=0 ;  ir1616n=0 ;  irs31no=0 ;   ir1616p=0    
      irp31po=0 ;   ir1616a=0 ;   irsi28ao=0 ;  ir1616d=0 ;   irp30do=0 ;  ir1216n=0 ;   irsi27no=0   
      ir1216p=0 ;   iral27po=0 ;  ir1216a=0 ;   irmg24ao=0 ;  irli7ng=0 ;  irli8gn=0 ;   irbe9pn=0    
      irb9np=0 ;    irb10an=0 ;   irn13na=0 ;   irb11an=0 ;   irn14na=0 ;  irn14pa=0 ;   irc11ap=0    
      irc11pg=0 ;   irn12gp=0 ;   irc12an=0 ;   iro15na=0 ;   irc13an=0 ;  iro16na=0 ;   irc13pn=0    
      irn13np=0 ;   irc14pg=0 ;   irn15gp=0 ;   irc14pn=0 ;   irn14np=0 ;  irc14ag=0 ;   iro18ga=0    
      iro16pa=0 ;   irn13ap=0 ;   irn14pn=0 ;   iro14np=0 ;   irn14an=0 ;  irf17na=0 ;   irn14ag=0    
      irf18ga=0 ;   irn15pn=0 ;   iro15np=0 ;   irn15an=0 ;   irf18na=0 ;  irn15ag=0 ;   irf19ga=0    
      iro14ag=0 ;   irne18ga=0 ;  iro17ag=0 ;   irne21ga=0 ;  iro17an=0 ;  irne20na=0 ;  iro18ag=0    
      irne22ga=0 ;  iro18an=0 ;   irne21na=0

      irne20pa=0 ;  irf17ap=0 ;   irf18pg=0 ;   irne19gp=0 ;  irf19pg=0 ;   irne20gp=0 ;  irf19pn=0    
      irne19np=0 ;  irf19ap=0 ;   irne22pa=0 ;  irna22na=0 ;  irf19an=0 ;   irna23pa=0 ;  irne20ap=0    
      irne20ng=0 ;  irne21gn=0 ;  irne21pg=0 ;  irna22gp=0 ;  irne21ag=0 ;  irmg25ga=0 ;  irne21an=0   
      irmg24na=0 ;  irne22pg=0 ;  irna23gp=0 ;  irne22ag=0 ;  irmg26ga=0 ;  irna22np=0 ;  irne22pn=0   
      irne22an=0 ;  irmg25na=0 ;  irna21pg=0 ;  irmg22gp=0 ;  irmg24pa=0 ;  irna21ap=0 ;  irna22pg=0   
      irmg23gp=0 ;  irna23pg=0 ;  irmg24gp=0 ;  irna23pn=0 ;  irmg23np=0 ;  irmg24pg=0 ;  iral25gp=0   
      iral27pa=0 ;  irmg24ap=0 ;  irmg25pg=0 ;  iral26gp=0 ;  irmg25ap=0 ;  iral28pa=0 ;  irmg25ag=0   
      irsi29ga=0 ;  irmg25an=0 ;  irsi28na=0 ;  irmg26pg=0 ;  iral27gp=0 ;  irmg26ag=0 ;  irsi30ga=0   
      irmg26an=0 ;  irsi29na=0 ;  iral25pg=0 ;  irsi26gp=0 ;  iral26pg=0 ;  irsi27gp=0 ;  iral27pg=0   
      irsi28gp=0 ;  iral27an=0 ;  irp30na=0 ;   irsi27pg=0 ;  irp28gp=0 ;   irsi28pg=0 ;  irp29gp=0    
      irsi29pg=0 ;  irp30gp=0 ;   irsi30pg=0 ;  irp31gp=0 ;   irr1=0 ;      irs1=0 ;      irt1=0       
      iru1=0 ;      irv1=0 ;      irw1=0 ;      irx1=0 ;      iry1=0 ;      ir1f54=0 ;    ir2f54=0     
      ir3f54=0 ;    ir4f54=0 ;    ir5f54=0 ;    ir6f54=0 ;    ir7f54=0 ;    ir8f54=0 ;    iralf1=0     
      iralf2=0 ;    irsi2ni=0 ;   irni2si=0 ;   iralam1=0 ;   irdelta1=0 ;  iralam2=0 ;   irdelta2=0 
      irdebug=0

      return
      end








      subroutine read_abundances_from_file(xin)
      include 'implno.dek'
      include 'network.dek'

! this routine reads a flat file of isotope names and mass fractions 

! declare the pass
      real*8            :: xin(*)


! for reading the abundance file
      character*80      :: string,word,atoken,aval
      logical           :: err
      integer           :: i,j,k,ip,getnam
      real*8            :: x,value2


! popular format statements
01    format(1x,a,a,a)
02    format(a)


! get file name
10    write(6,*) 'give abundance file name =>'
      read(5,2) string
      open(unit=2, file=string(1:len_trim(string)), status='old', err=10)


! for every line in the file
      do i=1,10000
       read(2,2,end=40) string

! replace any tab with a space
       j = len_trim(string)
       do k=1,j
        if (iachar(string(k:k)) .eq. 9) string(k:k) = ' '
       enddo

! strip leading blanks and any comments, which are preceeded by a ! or a #
       word = adjustl(string)
       j = len_trim(word)
       do k=1,j
        if (iachar(word(k:k)) .eq. 33 .or. iachar(word(k:k)) .eq. 35) then
         do ip=k,j
          word(ip:ip) = ' '
         end do
        end if
       end do

! get another line if this one is now empty 
       j = len_trim(word)
       if (j .eq. 0) cycle

!       write(6,*) trim(word)
!       read(5,*)

! chop the string to get the name and value
       ip = 1
       j = getnam(word,atoken,ip)
       j = getnam(word,aval,ip)
       x = value2(aval,err)
       if (err) then
        write(6,*)
        write(6,*) 'could not convert ',trim(aval),' to a number, skipping'
        write(6,*)
        goto 30
       end if

!       write(6,*) trim(atoken(1:5)),x
!       read(5,*)

! see if the isotope is in the network
       do j=1,ionmax
        if (ionam(j)(1:5) .eq. atoken(1:5)) then
         xin(j) = x
!         write(6,*) 'found it'
         goto 30
        end if
       enddo

! isotope not in network, check special cases
       if (atoken(1:5) .eq. 'n   ') then
        xin(ineut) = x
        goto 30
       else if (atoken(1:5) .eq. 'h1   ') then
        if (ih1 .ne. 0) xin(ih1) = x
        goto 30
       else if (atoken(1:5) .eq. 'p    ') then
        xin(iprot) = x
        goto 30
       else if (atoken(1:5) .eq. 'd    ') then
        if (ih2 .ne. 0) xin(ih2) = x
        goto 30
       else if (atoken(1:5) .eq. 't    ') then
        if (ih3 .ne. 0) xin(ih3) = x
        goto 30
       end if

! for sure not in the network list
       write(6,*)
       write(6,20) atoken(1:5),x
20     format(1x,'did not find isotope in network: ', a,' ',1pe13.5,' ,skipping it.')
       write(6,*)


! back for another line in the file 
30     continue
      enddo


! close file and return
40    close(unit=2)
      return 
      end
