! this file contains parts of the torch reaction network

      subroutine init_torch
      use weaktab
      include 'implno.dek'
      include 'const.dek'
      include 'network.dek'

! this routine initializes stuff for the torch network
!
! declare

      integer, parameter :: fil14=14
      real*8, parameter  :: mev2erg = ev2erg*1.0d6, mev2gr  = mev2erg/clight**2
      logical            :: ibhere
      character*132      :: string,word
      character*10       :: aname
      integer            :: i,j,k,l,ii,jj,ll,llmin,llmax,mm,j1,nz,na,nn,kk, &
                            inta,intz,izpf,iapf,inpf,nppf
      logical, parameter :: icoords = .true.
      real*8             :: dum,qful,xx


! popular format statements
01    format(a,i4)
06    format(2i5,f10.4)
07    format(f10.3)
08    format(6f10.3)
09    format(1x,i4,i4,i4,'    ',a5)



! zero all the isotope and rate pointers

      call init_isotope_rate_pointers


! decide on the arrow orientation
! downarrow true puts neutron, protons, alfa to the end
! downarrow false (i.e uparrow) puts neutron, protons, alfa at the beginning
! in general downarrowtrue  is faster for dense linear algebra, and either
! orientation for sparse linear algebra. gift routines, however, do
! much better with uparrow (downarrow false).

      downarrow = .true.
      if (downarrow) then
       ionbeg = 1
      else
       ionbeg = 4
      end if




! open the nuclear reaction rate data file

       if (which_bdat .eq. pre_jina_bdat) then
!        write(6,*) 'starting bdat read'
        open(unit=fil14,file='BDAT',status='old')
        nppf  = 5

       else if (which_bdat .eq. jina_bdat) then
!        write(6,*) 'starting bdatnew read'
        call read_partition_functions
        open(unit=fil14,file='BDATNEW',status='old')
        nppf  = 6
       else
        stop 'unknown which_bdat option in init_torch'
       end if


! put neutrons, protons and alfa first for up-arrow orientation
      if (.not.downarrow) then
       ineut        = 1
       aion(ineut)  = 1.0d0
       nion(ineut)  = 1.0d0
       zion(ineut)  = 0.0d0
       bion(ineut)  = 0.0d0
       mion(ineut)  = nion(ineut)*mn + zion(ineut)*(mp+me)  - bion(ineut)*mev2gr
       wion(ineut)  = avo * mion(ineut)
       ionam(ineut) = 'neut'

       iprot        = 2
       aion(iprot)  = 1.0d0
       nion(iprot)  = 0.0d0
       zion(iprot)  = 1.0d0
       bion(iprot)  = 0.0d0
       mion(iprot)  = nion(iprot)*mn + zion(iprot)*(mp+me) - bion(iprot)*mev2gr
       wion(iprot)  = avo * mion(iprot)
       ionam(iprot) = 'prot'

       ihe4         = 3
       aion(ihe4)   = 4.0d0
       nion(ihe4)   = 2.0d0
       zion(ihe4)   = 2.0d0
       bion(ihe4)   = 28.29603d0
       mion(ihe4)   = nion(ihe4)*mn + zion(ihe4)*(mp+me) - bion(ihe4)*mev2gr
       wion(ihe4)  = avo * mion(ihe4)
       ionam(ihe4)  = 'he4'
      end if


! now start reading the nuclear reaction rate data
! i is the code number of element z(i),n(i).
! forward reaction rates are 
! j = 1 = ng                 j = 6 = an
! j = 2 = pn                 j = 7  = ag
! j = 3 = ground state b-    j = 8 for semi-empirical electron capture
! j = 4 = pg                 j = 9 for semi-empirical positron decay
! j = 5 = ap                 j = 10 for semi-empirical beta decay
! ic1(j,i) = type formula to be used to calculate rate
! ic2(j,i) = number of constants in fitting reaction j on species i
! cx(k) holds the fitting coefficients
! ic3(j,i) = where to start counting cx from



! initialize counters for the ensuing goto loop construction
      k        = 1
      i        = ionbeg - 1


! we keep returning here from various goto and loop constructions
60    i = i+1
      if (i .gt. abignet) stop 'abignet too small in init_torch'
!      write(6,*) 'reading', i


! read in z and a and the two fitting information arrays
      read(fil14,02) nz,na,(ic1(j,i),ic2(j,i), j=1,10)
02    format(2i6,20i3)
!      write(6,*) 'read line 1', nz,na
      zion(i) = nz
      aion(i) = na
      if (nz .eq. 99 .and. na .eq. 99) go to 120


! read temperature dependent partition function coefficients
! not used now as the rauscher & thieleman table is interpolated

      llmin = nppf*(i-1) + 1
      llmax = llmin + (nppf - 1)
      if (llmax .gt. 6*abignet) stop 'array as past bounds'
      if (nppf .eq. 5) read(fil14,03) nz,nn,bion(i),(as(ll),ll=llmin,llmax),jj,aname
      if (nppf .eq. 6) read(fil14,04) nz,nn,bion(i),(as(ll),ll=llmin,llmax),jj,aname
03    format(2i3,f11.4,f5.1,4e12.3,i2,a10)
04    format(2i3,f11.4,f5.1,5e12.3,i2,a10)
      nion(i) = nn
      call sqeeze(aname)
!      write(6,*) 'read line 2 ',aname,' ',nn


! decide if this isotope is in the network and branch accordingly

! no he4 from jina reaclib/bdat
      if (nz .eq. 2 .and. na .eq. 4) goto 80

! no be8 from jina reaclib/bdat
      if (nz .eq. 4 .and. na .eq. 8) goto 80


      do jj=1,inetin
       if (int(zion(i)) .eq. izzz(jj)  .and. &
           int(nion(i)) .ge. inmin(jj) .and. &
           int(nion(i)) .le. inmax(jj)) goto 90
      enddo


! not using this isotope, but
! do the read, backup i by one, and go back to 60 for another isotope

80    continue
!      write(6,117) 'not using ',i,int(zion(i)),int(aion(i)),aname

      do jj=1,10
       if (ic1(jj,i) .gt. 0) then
        read(fil14,240) (dum,j1=1,ic2(jj,i))
240     format (7e13.6)
       end if
      enddo
      i = i - 1
      go to 60


! using this isotope, read coefficients of fitting formula into cx array
90    continue


!      write(6,117) 'using ',i,int(zion(i)),int(aion(i)),int(nion(i)),aname
117   format(1x,a,4i4,' ',a)


      do j=1,8
       ic3(j,i) = k
       if (ic1(j,i) .gt. 0) then
        kmax = k + ic2(j,i) - 1

        if (kmax .gt. cxdim) then
         write(6,*) 'kmax =', kmax,' cxdim =',cxdim
         stop 'kmax > cxdim in routine init_torch'
        end if

        read(fil14,240)  (cx(j1), j1=k,kmax)

!        write(6,*) j,ic1(j,i),ic2(j,i)
!        write(6,*) k,kmax,kmax-k+1
!        write(6,118) (cx(j1), j1=k,kmax)
!118     format(1x,1p7e16.8)

        k = kmax + 1
       end if
      enddo



! and go back for another isotope
      go to 60


! all done with the this part of the loading
120   continue
      close(unit=fil14)
!      write(6,*) 'done reading bdat'


! append neutrons, protons and alfa if down arrow
! set the ending isotope index ionend
      if (downarrow) then
       ionmax       = i
       ineut        = ionmax
       aion(ineut)  = 1.0d0
       nion(ineut)  = 1.0d0
       zion(ineut)  = 0.0d0
       bion(ineut)  = 0.0d0
       mion(ineut)  = nion(ineut)*mn + zion(ineut)*(mp+me) - bion(ineut)*mev2gr
       wion(ineut)  = avo * mion(ineut)
       ionam(ineut) = 'neut'
!       write(6,117) 'using ',ineut,int(zion(ineut)),int(aion(ineut)),int(nion(ineut)),ionam(ineut)

       ionmax       = ionmax + 1
       iprot        = ionmax
       aion(iprot)  = 1.0d0
       nion(iprot)  = 0.0d0
       zion(iprot)  = 1.0d0
       bion(iprot)  = 0.0d0
       mion(iprot)  = nion(iprot)*mn + zion(iprot)*(mp+me) - bion(iprot)*mev2gr
       wion(iprot)  = avo * mion(iprot)
       ionam(iprot) = 'prot'
!       write(6,117) 'using ',iprot,int(zion(iprot)),int(aion(iprot)),int(nion(iprot)),ionam(iprot)

       ionmax       = ionmax + 1
       ihe4         = ionmax
       aion(ihe4)   = 4.0d0
       nion(ihe4)   = 2.0d0
       zion(ihe4)   = 2.0d0
       bion(ihe4)   = 28.29603d0
       mion(ihe4)   = nion(ihe4)*mn + zion(ihe4)*(mp+me) - bion(ihe4)*mev2gr
       wion(ihe4)   = avo * mion(ihe4)
       ionam(ihe4)  = 'he4'
!       write(6,117) 'using ',ihe4,int(zion(ihe4)),int(aion(ihe4)),int(nion(ihe4)),ionam(ihe4)

       ionend       = ionmax - 3

! for up-arrow configurations
      else
       ionmax = i - 1
       ionend = ionmax
      end if


! for either orientation, append energy, temperature, and density pointers
       iener = ionmax + 1
       itemp = ionmax + 2
       iden  = ionmax + 3
       ivelx = ionmax + 4
       iposx = ionmax + 5
       neqs  = iposx

       ionam(iener) = 'ener '
       ionam(itemp) = 'temp '
       ionam(iden)  = 'den  '
       ionam(ivelx) = 'velx '
       ionam(iposx) = 'posx '


! number of neutrons
      nion(ionbeg:ionend) = aion(ionbeg:ionend) - zion(ionbeg:ionend)

! mass of each isotope assuming fully ionized
!      mion(ionbeg:ionend) = nion(ionbeg:ionend)*mn + zion(ionbeg:ionend)*(mp+me) - bion(ionbeg:ionend)*mev2gr
      mion(ionbeg:ionend) = nion(ionbeg:ionend)*mn + zion(ionbeg:ionend)*mp - bion(ionbeg:ionend)*mev2gr

! molar mass of each isotope
      wion(ionbeg:ionend) = avo * mion(ionbeg:ionend)

! here is a common approximation for the molar mass
      wion(ionbeg:ionmax) = aion(ionbeg:ionmax)




! build the links between the isotopes in the network

      call naray



! load the ffn, oda and langanke & martin-pinda weak rates

      call weaktab_init(ionmax,zion,aion)



! set the isotope names and pointers
!       write(6,*) ' '
!       write(6,*) ' using isotopes:'
!       write(6,*) '   i   z   a      name'

       if (.not.downarrow) then
        i    = ineut
        inta = aion(i)
        intz = zion(i)
!        write(6,09) ineut,intz,inta,ineut,ionam(i)

        i    = iprot
        inta = aion(i)
        intz = zion(i)
!        write(6,09) iprot,intz,inta,ionam(i)

        i    = ihe4
        inta = aion(i)
        intz = zion(i)
!        write(6,09) ihe4,intz,inta,ionam(i)
       endif


       do i=ionbeg,ionend
        inta = aion(i)
        intz = zion(i)
        if (intz .ge. 1  .and.  intz .le. zmax) then
         if (inta .ge. aidmin(intz)  .and.  inta .le. aidmax(intz)) then
          do ii = aidmin(intz),aidmax(intz)
           if (ii .eq. inta) then

            write(string,01) zsymb(intz),inta
            call sqeeze(string)
            ionam(i) = string

! using this isotope
!            write(6,09) i,intz,inta,ionam(i)

           end if
          enddo
         else
          write(6,*) ' bad aion',inta,' in routine init_torch'
          write(6,*) ' zion=',intz
          write(6,*) ' amin=',aidmin(intz),' amax=',aidmax(intz)
          stop 'error: bad inta in routine init_torch'
         end if
        else
         write(6,*) 'bad zion',intz,' in routine init_torch'
         write(6,*) 'inta =',inta,' zmax=', zmax
         stop 'error: bad intz in routine init_torch'
        end if
       enddo

       if (downarrow) then
        i    = ineut
        inta = aion(i)
        intz = zion(i)
!        write(6,09) ionmax-2,intz,inta,ionam(i)

        i    = iprot
        inta = aion(i)
        intz = zion(i)
!        write(6,09) ionmax-1,intz,inta,ionam(i)

        i    = ihe4
        inta = aion(i)
        intz = zion(i)
!        write(6,09) ionmax,intz,inta,ionam(i)
       endif



! write a isotope file - useful for plotting the network
       if (icoords) then
        open(unit=44,file='isotope_listing.dat',status='unknown')
        do i=1,ionmax
         write(44,'(1x,3i4,a,a)') int(zion(i)), int(aion(i)), int(nion(i)),' ',ionam(i)
        enddo
        close(unit=44)
        write(6,*) 'wrote file isotope_listing.dat'
        write(6,*)
       end if

!      do i=1,ionmax
!       write(6,888) ionam(i), &
!                   int(zion(i)),int(nion(i)),int(aion(i)), &
!                   mion(i)*avo,(mion(i)*avo-aion(i)), &
!                   (mion(i)*avo-aion(i))/(mev2gr*avo)
! 888   format(1x,a,3i4,1p5e18.10)
!      enddo
!      read(5,*)



! set the id numbers of certain key isotopes
       do i=ionbeg,ionend
        if (ionam(i) .eq. 'h2   ')   then
         ih2 = i
        else if (ionam(i) .eq. 'h3   ') then
         ih3 = i
        else if (ionam(i) .eq. 'he3  ') then
         ihe3 = i
        else if (ionam(i) .eq. 'li6  ') then
         ili6 = i
        else if (ionam(i) .eq. 'li7  ') then
         ili7 = i
        else if (ionam(i) .eq. 'li8  ') then
         ili8 = i
        else if (ionam(i) .eq. 'be7  ') then
         ibe7 = i
        else if (ionam(i) .eq. 'be9  ') then
         ibe9 = i
        else if (ionam(i) .eq. 'b8   ') then
         ib8 = i
        else if (ionam(i) .eq. 'b9   ') then
         ib9 = i
        else if (ionam(i) .eq. 'b10  ') then
         ib10 = i
        else if (ionam(i) .eq. 'b11  ') then
         ib11 = i
        else if (ionam(i) .eq. 'c11  ') then
         ic11 = i
        else if (ionam(i) .eq. 'c12  ') then
         ic12 = i
        else if (ionam(i) .eq. 'c13  ') then
         ic13 = i
        else if (ionam(i) .eq. 'c14  ') then
         ic14 = i
        else if (ionam(i) .eq. 'n12  ') then
         in12 = i
        else if (ionam(i) .eq. 'n13  ') then
         in13 = i
        else if (ionam(i) .eq. 'n14  ') then
         in14 = i
        else if (ionam(i) .eq. 'n15  ') then
         in15 = i
        else if (ionam(i) .eq. 'o14  ') then
         io14 = i
        else if (ionam(i) .eq. 'o15  ') then
         io15 = i
        else if (ionam(i) .eq. 'o16  ') then
         io16 = i
        else if (ionam(i) .eq. 'o17  ') then
         io17 = i
        else if (ionam(i) .eq. 'o18  ') then
         io18 = i
        else if (ionam(i) .eq. 'f17  ') then
         if17 = i
        else if (ionam(i) .eq. 'f18  ') then
         if18 = i
        else if (ionam(i) .eq. 'f19  ') then
         if19 = i
        else if (ionam(i) .eq. 'ne18 ') then
         ine18 = i
        else if (ionam(i) .eq. 'ne19 ') then
         ine19 = i
        else if (ionam(i) .eq. 'ne20 ') then
         ine20 = i
        else if (ionam(i) .eq. 'ne21 ') then
         ine21 = i
        else if (ionam(i) .eq. 'ne22 ') then
         ine22 = i
        else if (ionam(i) .eq. 'ne23 ') then
         ine23 = i
        else if (ionam(i) .eq. 'na20 ') then
         ina20 = i
        else if (ionam(i) .eq. 'na21 ') then
         ina21 = i
        else if (ionam(i) .eq. 'na22 ') then
         ina22 = i
        else if (ionam(i) .eq. 'na23 ') then
         ina23 = i
        else if (ionam(i) .eq. 'mg22 ') then
         img22 = i
        else if (ionam(i) .eq. 'mg23 ') then
         img23 = i
        else if (ionam(i) .eq. 'mg24 ') then
         img24 = i
        else if (ionam(i) .eq. 'mg25 ') then
         img25 = i
        else if (ionam(i) .eq. 'mg26 ') then
         img26 = i
        else if (ionam(i) .eq. 'al25 ') then
         ial25 = i
        else if (ionam(i) .eq. 'al26 ') then
         ial26 = i
        else if (ionam(i) .eq. 'al27 ') then
         ial27 = i
        else if (ionam(i) .eq. 'si27 ') then
         isi27 = i
        else if (ionam(i) .eq. 'si28 ') then
         isi28 = i
        else if (ionam(i) .eq. 'si29 ') then
         isi29 = i
        else if (ionam(i) .eq. 'si30 ') then
         isi30 = i
        else if (ionam(i) .eq. 'p30  ') then
         ip30  = i
        else if (ionam(i) .eq. 'p31  ') then
         ip31  = i
        else if (ionam(i) .eq. 's30  ') then
         is30  = i
        else if (ionam(i) .eq. 's31  ') then
         is31  = i
        else if (ionam(i) .eq. 's32  ') then
         is32  = i
        else if (ionam(i) .eq. 'cl35 ') then
         icl35  = i
        else if (ionam(i) .eq. 'ar36 ') then
         iar36  = i
        else if (ionam(i) .eq. 'ar37 ') then
         iar37  = i
        else if (ionam(i) .eq. 'ar38 ') then
         iar38  = i
        else if (ionam(i) .eq. 'k39  ') then
         ik39  = i
        else if (ionam(i) .eq. 'ca40 ') then
         ica40  = i
        else if (ionam(i) .eq. 'sc43 ') then
         isc43  = i
        else if (ionam(i) .eq. 'sc45 ') then
         isc45  = i
        else if (ionam(i) .eq. 'ti44 ') then
         iti44  = i
        else if (ionam(i) .eq. 'ti46 ') then
         iti46  = i
        else if (ionam(i) .eq. 'ti48 ') then
         iti48  = i
        else if (ionam(i) .eq. 'ti50 ') then
         iti50  = i
        else if (ionam(i) .eq. 'v46  ') then
         iv46  = i
        else if (ionam(i) .eq. 'v47  ') then
         iv47  = i
        else if (ionam(i) .eq. 'v48  ') then
         iv48  = i
        else if (ionam(i) .eq. 'v51  ') then
         iv51  = i
        else if (ionam(i) .eq. 'cr47 ') then
         icr47  = i
        else if (ionam(i) .eq. 'cr48 ') then
         icr48  = i
        else if (ionam(i) .eq. 'cr49 ') then
         icr49  = i
        else if (ionam(i) .eq. 'cr50 ') then
         icr50  = i
        else if (ionam(i) .eq. 'cr51 ') then
         icr51  = i
        else if (ionam(i) .eq. 'cr52 ') then
         icr52  = i
        else if (ionam(i) .eq. 'cr53 ') then
         icr53  = i
        else if (ionam(i) .eq. 'cr54 ') then
         icr54  = i
        else if (ionam(i) .eq. 'mn50 ') then
         imn50  = i
        else if (ionam(i) .eq. 'mn51 ') then
         imn51  = i
        else if (ionam(i) .eq. 'mn52 ') then
         imn52  = i
        else if (ionam(i) .eq. 'mn55 ') then
         imn55  = i
        else if (ionam(i) .eq. 'fe52 ') then
         ife52 = i
        else if (ionam(i) .eq. 'fe54 ') then
         ife54 = i
        else if (ionam(i) .eq. 'fe55 ') then
         ife55 = i
        else if (ionam(i) .eq. 'fe56 ') then
         ife56 = i
        else if (ionam(i) .eq. 'fe57 ') then
         ife57 = i
        else if (ionam(i) .eq. 'fe58 ') then
         ife58 = i
        else if (ionam(i) .eq. 'fe59 ') then
         ife59 = i
        else if (ionam(i) .eq. 'co54 ') then
         ico54 = i
        else if (ionam(i) .eq. 'co55 ') then
         ico55 = i
        else if (ionam(i) .eq. 'co56 ') then
         ico56 = i
        else if (ionam(i) .eq. 'co59 ') then
         ico59 = i
        else if (ionam(i) .eq. 'ni56 ') then
         ini56 = i
        else if (ionam(i) .eq. 'ni57 ') then
         ini57 = i
        else if (ionam(i) .eq. 'ni58 ') then
         ini58 = i
        else if (ionam(i) .eq. 'ni59 ') then
         ini59 = i
        else if (ionam(i) .eq. 'ni64 ') then
         ini64 = i
        else if (ionam(i) .eq. 'ni66 ') then
         ini66 = i
        else if (ionam(i) .eq. 'cu63 ') then
         icu63 = i
        else if (ionam(i) .eq. 'zn60 ') then
         izn60 = i
        else if (ionam(i) .eq. 'zn64 ') then
         izn64 = i
        end if
       enddo




! now set reaction rate names and pointers

       if (icoords) open(unit=44,file='isotope_linkages.dat',status='unknown')

       nrat = 0
       ratnam(1:abigrat) = ' ' 
       do j=ionbeg,ionend

        if (nrat+14 .gt. abigrat) stop 'nrat > abigrat in init_torch'

! set up the (n,g) and (g,n) names and a few reaction rate pointers
        k = nrr(1,j)
        if (k .gt. 0) then

         if (icoords) call coords(j,k)

         string = 'r_'//ionam(j)//'_ng_'//ionam(k)
!         string = ionam(j)//'(n,g)'//ionam(k)
!         string = 'n '//ionam(j)//' g '//ionam(k)

         call sqeeze(string)
         nrat = nrat + 1
         ratnam(nrat) = string

         string = 'r_'//ionam(k)//'_gn_'//ionam(j)
!         string = ionam(k)//'(g,n)'//ionam(j)
!         string = 'g '//ionam(k)//' n '//ionam(j)

         call sqeeze(string)
         nrat = nrat + 1
         ratnam(nrat) = string

         if (j .eq. ih2   .and. k .eq. ih3)  then
          irdng = nrat-1
          irtgn = nrat
         else if (j .eq. ili6  .and. k .eq. ili7) then
          irli6ng = nrat-1
          irli7gn = nrat
         else if (j .eq. ili7  .and. k .eq. ili8) then
          irli7ng = nrat-1
          irli8gn = nrat
         else if (j .eq. ine20  .and. k .eq. ine21) then
          irne20ng = nrat-1
          irne21gn = nrat
         end if
        end if


! set up the (p,n) (n,p) beta- beta+ decay names and a few reaction rate pointers
        k = nrr(2,j)
        if (k .gt. 0) then

         if (icoords) call coords(j,k)

         string = 'r_'//ionam(j)//'_pn_'//ionam(k)
!         string = ionam(j)//'(p,n)'//ionam(k)
!         string = 'p '//ionam(j)//' n '//ionam(k)

         call sqeeze(string)
         nrat = nrat + 1
         ratnam(nrat) = string


         string = 'r_'//ionam(k)//'_np_'//ionam(j)
!         string = ionam(k)//'(n,p)'//ionam(j)
!         string = 'n '//ionam(k)//' p '//ionam(j)

         call sqeeze(string)
         nrat = nrat + 1
         ratnam(nrat) = string

         if (j .eq. ih3   .and. k .eq. ihe3) then
          irtpn = nrat - 1
          irhe3np = nrat
         else if (j .eq. ili7  .and. k .eq. ibe7) then
          irli7pn = nrat - 1
          irbe7np = nrat
         else if (j .eq. ibe9  .and. k .eq. ib9)  then
          irbe9pn = nrat - 1
          irb9np = nrat
         else if (j .eq. ib11  .and. k .eq. ic11) then
          irb11pn = nrat - 1
          irc11np = nrat
         else if (j .eq. ic13  .and. k .eq. in13) then
          irc13pn = nrat - 1
          irn13np = nrat
         else if (j .eq. ic14  .and. k .eq. in14) then
          irc14pn = nrat - 1
          irn13np = nrat
         else if (j .eq. in14  .and. k .eq. io14) then
          irn14pn = nrat - 1
          iro14np = nrat
         else if (j .eq. in15  .and. k .eq. io15) then
          irn15pn = nrat - 1
          iro15np = nrat
         else if (j .eq. if19  .and. k .eq. ine19) then
          irf19pn = nrat - 1
          irne19np = nrat
         else if (j .eq. ine22  .and. k .eq. ina22) then
          irne22pn = nrat - 1
          irna22np = nrat
         else if (j .eq. ina23  .and. k .eq. img23) then
          irna23pn = nrat - 1
          irmg23np = nrat
         end if

         string = 'r_'//ionam(j)//'_wk-minus_'//ionam(k)
!         string = ionam(j)//'(n=>p)'//ionam(k)
!         string = ionam(j)//' e+ + nub '//ionam(k)

         call sqeeze(string)
         nrat = nrat + 1
         ratnam(nrat) = string

         if (j .eq. ibe7   .and. k .eq. ili7) irbeec = nrat

         string = 'r_'//ionam(k)//'_wk_'//ionam(j)
!         string = ionam(k)//'(p=>n)'//ionam(j)
!         string = ionam(k)//' e- + nu '//ionam(j)

         call sqeeze(string)
         nrat = nrat + 1
         ratnam(nrat) = string

        end if


! set up the (p,g) and (g,p) names and a few reaction rate pointers
        k = nrr(3,j)
        if (k .gt. 0) then

         if (icoords) call coords(j,k)

         string = 'r_'//ionam(j)//'_pg_'//ionam(k)
!         string = ionam(j)//'(p,g)'//ionam(k)
!         string = 'p '//ionam(j)//' g '//ionam(k)

         call sqeeze(string)
         nrat = nrat + 1
         ratnam(nrat) = string

         string = 'r_'//ionam(k)//'_gp_'//ionam(j)
!         string = ionam(k)//'(g,p)'//ionam(j)
!         string = 'g '//ionam(k)//' p '//ionam(j)

         call sqeeze(string)
         nrat = nrat + 1
         ratnam(nrat) = string

         if (j .eq. ih2   .and. k .eq. ihe3) then
          irdpg = nrat - 1
          irhe3gp = nrat
         else if (j .eq. ili6  .and. k .eq. ibe7) then
          irli6pg = nrat - 1
          irbe7gp = nrat
         else if (j .eq. ibe7  .and. k .eq. ib8)  then
          irbe7pg = nrat - 1
          irb8gp = nrat
         else if (j .eq. ibe9  .and. k .eq. ib10) then
          irbe9pg = nrat - 1
          irb10gp = nrat
         else if (j .eq. ib10  .and. k .eq. ic11) then
          irb10pg = nrat - 1
          irc11gp = nrat
         else if (j .eq. ib11  .and. k .eq. ic12) then
          irb11pg = nrat - 1
          irc12gp = nrat
         else if (j .eq. ic11  .and. k .eq. in12) then
          irc11pg = nrat - 1
          irn12gp = nrat
         else if (j .eq. ic14  .and. k .eq. in15) then
          irc14pg = nrat - 1
          irn15gp = nrat
         else if (j .eq. in13  .and. k .eq. io14) then
          irn13pg = nrat - 1
          iro14gp = nrat
         else if (j .eq. in14  .and. k .eq. io15) then
          irn14pg = nrat - 1
          iro15gp = nrat
         else if (j .eq. in15  .and. k .eq. io16) then
          irn15pg = nrat - 1
          iro16gp = nrat
         else if (j .eq. io16  .and. k .eq. if17) then
          iro16pg = nrat - 1
          irf17gp = nrat
         else if (j .eq. io17  .and. k .eq. if18) then
          iro17pg = nrat - 1
          irf18gp = nrat
         else if (j .eq. io18  .and. k .eq. if19) then
          iro18pg = nrat - 1
          irf19gp = nrat
         else if (j .eq. if17  .and. k .eq. ine18) then
          irf17pg  = nrat - 1
          irne18gp = nrat
         else if (j .eq. if18  .and. k .eq. ine19) then
          irf18pg  = nrat - 1
          irne19gp = nrat
         else if (j .eq. if19  .and. k .eq. ine20) then
          irf19pg  = nrat - 1
          irne20gp = nrat
         else if (j .eq. ine19  .and. k .eq. ina20) then
          irne19pg = nrat - 1
          irna20gp = nrat
         else if (j .eq. ine20  .and. k .eq. ina21) then
          irne20pg = nrat - 1
          irna21gp = nrat
         else if (j .eq. ine21  .and. k .eq. ina22) then
          irne21pg = nrat - 1
          irna22gp = nrat
         else if (j .eq. ine22  .and. k .eq. ina23) then
          irne22pg = nrat - 1
          irna23gp = nrat
         else if (j .eq. ina21  .and. k .eq. img22) then
          irna21pg = nrat - 1
          irmg22gp = nrat
         else if (j .eq. ina22  .and. k .eq. img23) then
          irna22pg = nrat - 1
          irmg23gp = nrat
         else if (j .eq. ina23  .and. k .eq. img24) then
          irna23pg = nrat - 1
          irmg24gp = nrat
         else if (j .eq. img24  .and. k .eq. ial25) then
          irmg24pg = nrat - 1
          iral25gp = nrat
         else if (j .eq. img25  .and. k .eq. ial26) then
          irmg25pg = nrat - 1
          iral26gp = nrat
         else if (j .eq. img26  .and. k .eq. ial27) then
          irmg26pg = nrat - 1
          iral27gp = nrat
         else if (j .eq. ial25  .and. k .eq. isi26) then
          iral25pg = nrat - 1
          irsi26gp = nrat
         else if (j .eq. ial26  .and. k .eq. isi27) then
          iral26pg = nrat - 1
          irsi27gp = nrat
         else if (j .eq. ial27  .and. k .eq. isi28) then
          iral27pg = nrat - 1
          irsi28gp = nrat
         else if (j .eq. isi27  .and. k .eq. ip28) then
          irsi27pg = nrat - 1
          irp28gp = nrat
         else if (j .eq. isi28  .and. k .eq. ip29) then
          irsi28pg = nrat - 1
          irp29gp = nrat
         else if (j .eq. isi29  .and. k .eq. ip30) then
          irsi29pg = nrat - 1
          irp30gp = nrat
         else if (j .eq. isi30  .and. k .eq. ip31) then
          irsi30pg = nrat - 1
          irp31gp = nrat
         end if
        end if


! set up the (a,p) and (p,a) names and a few reaction rate pointers
        k = nrr(4,j)
        if (k .gt. 0) then

         if (icoords) call coords(j,k)

         string = 'r_'//ionam(j)//'_ap_'//ionam(k)
!         string = ionam(j)//'(a,p)'//ionam(k)
!         string = 'he4 '//ionam(j)//' p '//ionam(k)

         call sqeeze(string)
         nrat = nrat + 1
         ratnam(nrat) = string

         string = 'r_'//ionam(k)//'_pa_'//ionam(j)
!         string = ionam(k)//'(p,a)'//ionam(j)
!         string = 'p '//ionam(k)//' he4 '//ionam(j)

         call sqeeze(string)
         nrat = nrat + 1
         ratnam(nrat) = string

         if (j .eq. ihe3  .and. k .eq. ili6) then
          irhe3ap = nrat - 1
          irli6pa = nrat
         else if (j .eq. ili6  .and. k .eq. ibe9) then
          irli6ap = nrat - 1
          irbe9pa = nrat
         else if (j .eq. ibe7  .and. k .eq. ib10) then
          irbe7ap = nrat - 1
          irb10pa = nrat
         else if (j .eq. ib8   .and. k .eq. ic11) then
          irb8ap = nrat - 1
          irc11pa = nrat
         else if (j .eq. ib11  .and. k .eq. ic14) then
          irb11ap = nrat - 1
          irc14pa = nrat
         else if (j .eq. ic11  .and. k .eq. in14) then
          irc11ap = nrat - 1
          irn14pa = nrat
         else if (j .eq. in13  .and. k .eq. io16) then
          irn13ap = nrat - 1
          iro16pa = nrat
         else if (j .eq. in14  .and. k .eq. io17) then
          irn14ap = nrat - 1
          iro17pa = nrat
         else if (j .eq. in15  .and. k .eq. io18) then
          irn15ap = nrat - 1
          iro18pa = nrat
         else if (j .eq. io14  .and. k .eq. if17) then
          iro14ap = nrat - 1
          irf17pa = nrat
         else if (j .eq. io15  .and. k .eq. if18) then
          iro15ap = nrat - 1
          irf18pa = nrat
         else if (j .eq. io16  .and. k .eq. if19) then
          iro16ap = nrat - 1
          irf19pa = nrat
         else if (j .eq. if17  .and. k .eq. ine20) then
          irf17ap  = nrat - 1
          irne20pa = nrat
         else if (j .eq. if19  .and. k .eq. ine22) then
          irf19ap  = nrat - 1
          irne22pa = nrat
         else if (j .eq. ine20  .and. k .eq. ina23) then
          irne20ap  = nrat - 1
          irna23pa  = nrat
         else if (j .eq. ina21  .and. k .eq. img24) then
          irna21ap  = nrat - 1
          irmg24pa  = nrat
         else if (j .eq. img24  .and. k .eq. ial27) then
          irmg24ap  = nrat - 1
          iral27pa  = nrat
         else if (j .eq. img25  .and. k .eq. ial28) then
          irmg25ap  = nrat - 1
          iral28pa  = nrat
         end if
        end if


! set up the (a,n) and (n,a) names and a few reaction rate pointers
        k = nrr(5,j)
        if (k .gt. 0) then

         if (icoords) call coords(j,k)

         string = 'r_'//ionam(j)//'_an_'//ionam(k)
!         string = ionam(j)//'(a,n)'//ionam(k)
!         string = 'he4 '//ionam(j)//' n '//ionam(k)

         call sqeeze(string)
         nrat = nrat + 1
         ratnam(nrat) = string

         string = 'r_'//ionam(k)//'_na_'//ionam(j)
!         string = ionam(k)//'(n,a)'//ionam(j)
!         string = 'n '//ionam(k)//' he4 '//ionam(j)

         call sqeeze(string)
         nrat = nrat + 1
         ratnam(nrat) = string

         if (j .eq. ih3   .and. k .eq. ili6) then
          irtan = nrat - 1
          irli6na = nrat
         else if (j .eq. ili7  .and. k .eq. ib10) then
          irli7an = nrat - 1
          irb10na = nrat
         else if (j .eq. ibe9  .and. k .eq. ic12) then
          irbe9an = nrat - 1
          irc12na = nrat
         else if (j .eq. ib10  .and. k .eq. in13) then
          irb10an = nrat - 1
          irn13na = nrat
         else if (j .eq. ib11  .and. k .eq. in14) then
          irb11an = nrat - 1
          irn14na = nrat
         else if (j .eq. ic12  .and. k .eq. io15) then
          irc12an = nrat - 1
          iro15na = nrat
         else if (j .eq. ic13  .and. k .eq. io16) then
          irc13an = nrat - 1
          iro16na = nrat
         else if (j .eq. in14  .and. k .eq. if17) then
          irn14an = nrat - 1
          irf17na = nrat
         else if (j .eq. in15  .and. k .eq. if18) then
          irn15an = nrat - 1
          irf18na = nrat
         else if (j .eq. io17  .and. k .eq. ine20) then
          iro17an = nrat - 1
          irne20na = nrat
         else if (j .eq. io18  .and. k .eq. ine21) then
          iro18an = nrat - 1
          irne21na = nrat
         else if (j .eq. if19  .and. k .eq. ina22) then
          irf19an  = nrat - 1
          irna22na = nrat
         else if (j .eq. ine21  .and. k .eq. img24) then
          irne21an  = nrat - 1
          irmg24na  = nrat
         else if (j .eq. ine22  .and. k .eq. img25) then
          irne22an  = nrat - 1
          irmg25na  = nrat
         else if (j .eq. img25  .and. k .eq. isi28) then
          irmg25an  = nrat - 1
          irsi28na  = nrat
         else if (j .eq. img26  .and. k .eq. isi29) then
          irmg26an  = nrat - 1
          irsi29na  = nrat
         else if (j .eq. ial27  .and. k .eq. ip30) then
          iral27an  = nrat - 1
          irp30na  = nrat
         end if
        end if


! and the (a,g) and (g,a) names and a few reaction rate pointers
        k = nrr(6,j)
        if (k .gt. 0) then

         if (icoords) call coords(j,k)

         string = 'r_'//ionam(j)//'_ag_'//ionam(k)
!         string = ionam(j)//'(a,g)'//ionam(k)
!         string = 'he4 '//ionam(j)//' g '//ionam(k)

         call sqeeze(string)
         nrat = nrat + 1
         ratnam(nrat) = string

         string = 'r_'//ionam(k)//'_ga_'//ionam(j)
!         string = ionam(k)//'(g,a)'//ionam(j)
!         string = 'g '//ionam(k)//' he4 '//ionam(j)

         call sqeeze(string)
         nrat = nrat + 1
         ratnam(nrat) = string

         if (j .eq. ih2   .and. k .eq. ili6) then
          irdag = nrat - 1
          irli6ga = nrat
         else if (j .eq. ih3   .and. k .eq. ili7) then
          irtag = nrat - 1
          irli7ga = nrat
         else if (j .eq. ihe3  .and. k .eq. ibe7) then
          irhe3ag = nrat - 1
          irbe7ga = nrat
         else if (j .eq. ili6  .and. k .eq. ib10) then
          irli6ag = nrat - 1
          irb10ga = nrat
         else if (j .eq. ili7  .and. k .eq. ib11) then
          irli7ag = nrat - 1
          irb11ga = nrat
         else if (j .eq. ibe7  .and. k .eq. ic11) then
          irbe7ag = nrat - 1
          irc11ga = nrat
         else if (j .eq. ic12  .and. k .eq. io16) then
          ircag = nrat - 1
          iroga = nrat
         else if (j .eq. ic14  .and. k .eq. io18) then
          irc14ag = nrat - 1
          iro18ga = nrat
         else if (j .eq. in14  .and. k .eq. if18) then
          irn14ag = nrat - 1
          irf18ga = nrat
         else if (j .eq. in15  .and. k .eq. if19) then
          irn15ag = nrat - 1
          irf19ga = nrat
         else if (j .eq. io14  .and. k .eq. ine18) then
          iro14ag = nrat - 1
          irne18ga = nrat
         else if (j .eq. io15  .and. k .eq. ine19) then
          iro15ag = nrat - 1
          irne19ga = nrat
         else if (j .eq. io16  .and. k .eq. ine20) then
          iroag = nrat - 1
          irnega = nrat
         else if (j .eq. io17  .and. k .eq. ine21) then
          iro17ag = nrat - 1
          irne21ga = nrat
         else if (j .eq. io18  .and. k .eq. ine22) then
          iro18ag = nrat - 1
          irne22ga = nrat
         else if (j .eq. ine20  .and. k .eq. img24) then
          irneag = nrat - 1
          irmgga = nrat
         else if (j .eq. ine21  .and. k .eq. img25) then
          irne21ag = nrat - 1
          irmg25ga = nrat
         else if (j .eq. ine22  .and. k .eq. img26) then
          irne22ag = nrat - 1
          irmg26ga = nrat
         else if (j .eq. img24  .and. k .eq. isi28) then
          irmgag = nrat - 1
          irsiga = nrat
         else if (j .eq. img25  .and. k .eq. isi29) then
          irmg25ag = nrat - 1
          irsi29ga = nrat
         else if (j .eq. img26  .and. k .eq. isi30) then
          irmg26ag = nrat - 1
          irsi30ga = nrat
         else if (j .eq. isi28  .and. k .eq. is32) then
          irsiag = nrat - 1
          irsga  = nrat
         end if
        end if
       enddo


! make sure we have the space before adding in the hardcoded rates

       if (nrat + 67 .gt. abigrat) stop 'nrat > abigrat in init_torch'


! for p(e-,nu)n and n(e+,nub)p reactions
       nrat  = nrat + 1
       irpen = nrat
       ratnam(irpen) = 'r_h1_wk_neut'

       nrat = nrat + 1
       irnep = nrat
       ratnam(irnep) = 'r_neut_wk-minus_h1'

       if (icoords) call coords(ineut,iprot)



! c12 reactions, first half of triple alpha
       if (ic12 .ne. 0) then
        nrat = nrat + 1
        ir3a = nrat
        ratnam(ir3a) = 'r_he4_he4_he4_to_c12'

        nrat = nrat + 1
        irg3a = nrat
        ratnam(irg3a) = 'r_c12_to_he4_he4_he4'

        if (icoords) call coords(ihe4,ic12)


! c12+c12 reactions; must have ne20, na23, mg23 in the network
        if (ine20 .ne. 0  .and. ina23 .ne. 0  .and.  img23 .ne. 0) then

         nrat = nrat + 1
         ir1212n = nrat
         ratnam(ir1212n) = 'r_c12_c12_to_neut_mg23'

         nrat = nrat + 1
         irmg23nc = nrat
         ratnam(irmg23nc) = 'r_neut_mg23_to_c12_c12'

         nrat = nrat + 1
         ir1212p = nrat
         ratnam(ir1212p) = 'r_c12_c12_to_h1_na23'

         nrat = nrat + 1
         irna23pc = nrat
         ratnam(irna23pc) = 'r_h1_na23_to_c12_c12'

         nrat = nrat + 1
         ir1212a = nrat
         ratnam(ir1212a) = 'r_c12_c12_to_he4_ne20'

         nrat = nrat + 1
         irne20ac = nrat
         ratnam(irne20ac) = 'r_he4_ne20_to_c12_c12'

         if (icoords) then
          call coords(ic12,ine20)
          call coords(ic12,ina23)
          call coords(ic12,img23)
         end if

        end if
       end if


! o16+o16 reactions; must have si28, p30, p31 and s31 in the network
       if (io16 .ne. 0 .and. isi28 .ne. 0 .and. ip30 .ne. 0 .and. ip31 .ne. 0 .and. is31 .ne. 0) then

        nrat = nrat + 1
        ir1616n = nrat
        ratnam(nrat) = 'r_o16_o16_to_neut_s31'

        nrat = nrat + 1
        irs31no = nrat
        ratnam(irs31no) = 'r_neut_s31_to_o16_o16'

        nrat = nrat + 1
        ir1616p = nrat
        ratnam(ir1616p) = 'r_o16_o16_to_h1_p31'

        nrat = nrat + 1
        irp31po = nrat
        ratnam(irp31po) = 'r_h1_p31_to_o16_o16'

        nrat = nrat + 1
        ir1616a = nrat
        ratnam(ir1616a) = 'r_o16_o16_to_he4_si28'

        nrat = nrat + 1
        irsi28ao = nrat
        ratnam(irsi28ao) = 'r_he4_si28_to_o16_o16'

        nrat = nrat + 1
        ir1616d = nrat
        ratnam(ir1616d) = 'r_o16_o16_to_h2_p30'

        nrat = nrat + 1
        irp30do = nrat
        ratnam(irp30do) = 'r_h2_p30_to_o16_o16'

         if (icoords) then
          call coords(io16,isi28)
          call coords(io16,ip30)
          call coords(io16,ip31)
          call coords(io16,is31)
         end if
       end if


! c12+o16 reactions; must have mg24, al27, si27 in the network
       if (ic12 .ne. 0 .and. io16 .ne. 0 .and. img24 .ne. 0  .and. ial27 .ne. 0  .and.  isi27 .ne. 0) then

        nrat = nrat + 1
        ir1216n = nrat
        ratnam(ir1216n) = 'r_c12_o16_to_neut_si27'

        nrat = nrat + 1
        irsi27no = nrat
        ratnam(irsi27no) = 'r_neut_si27_to_c12_o16'

        nrat = nrat + 1
        ir1216p = nrat
        ratnam(ir1216p) = 'r_c12_o16_to_h1_al27'

        nrat = nrat + 1
        iral27po = nrat
        ratnam(iral27po) = 'r_h1_al27_to_c12_o16'

        nrat = nrat + 1
        ir1216a = nrat
        ratnam(ir1216a) = 'r_c12_o16_to_he4_mg24'

        nrat = nrat + 1
        irmg24ao = nrat
        ratnam(irmg24ao) = 'r_he4_mg24_to_c12_o16'

        if (icoords) then
         call coords(ic12,io16)
         call coords(ic12,img24)
         call coords(ic12,ial27)
         call coords(ic12,isi27)
         call coords(io16,img24)
         call coords(io16,ial27)
         call coords(io16,isi27)
        end if
       end if


! if we have deuterium
       if (ih2 .ne. 0) then

! pp
        nrat = nrat + 1
        irpp = nrat
        ratnam(irpp) = 'r_h1_h1_wk_h2'

        nrat = nrat + 1
        irpep = nrat
        ratnam(irpep) = 'r_h1_h1_ec_h2'

! p(n,g)d
        nrat = nrat + 1
        irpng = nrat
        ratnam(irpng) = 'r_h1_neut_to_h2'

        nrat = nrat + 1
        irdgn = nrat
        ratnam(irdgn) = 'r_h2_to_h1_neut'

        if (icoords) then
         call coords(iprot,ih2)
         call coords(ineut,ih2)
        end if


! d(p,n)2p
        nrat = nrat + 1
        irdpn = nrat
        ratnam(irdpn) = 'r_h2_h1_to_neut_h1_h1'

        nrat = nrat + 1
        ir2pnp = nrat
        ratnam(ir2pnp) = 'r_h1_h1_neut_to_h1_h2'

! d(d,g)he4
        nrat = nrat + 1
        irddg = nrat
        ratnam(irddg) = 'r_h2_h2_to_he4'

        nrat = nrat + 1
        irhe4gd = nrat
        ratnam(irhe4gd) = 'r_he4_to_h2_h2'

        if (icoords) call coords(ih2,ihe4)

       end if


! if we have deuterium and tritium
       if (ih2 .ne. 0  .and. ih3 .ne. 0) then

! t(d,n)he4 dt reaction
        nrat = nrat + 1
        irtdn = nrat
        ratnam(irtdn) = 'r_h2_h3_to_neut_he4'

        nrat = nrat + 1
        irhe4nd = nrat
        ratnam(irhe4nd) = 'r_neut_he4_to_h2_h3'

        if (icoords) then
         call coords(ineut,ih2)
         call coords(ineut,ih3)
         call coords(ineut,ihe4)
         call coords(ih2,ih3)
         call coords(ih2,ihe4)
         call coords(ih3,ihe4)
        end if

! d(d,p)t
        nrat = nrat + 1
        irddp = nrat
        ratnam(irddp) = 'r_h2_h2_to_h1_h3'

        nrat = nrat + 1
        irtpd = nrat
        ratnam(irtpd) = 'r_h1_h3_to_h2_h2'

        if (icoords) then
         call coords(iprot,ih2)
         call coords(iprot,ih3)
        end if

       end if


! if we have tritium
       if (ih3 .ne. 0) then

! t(p,g)he4
        nrat = nrat + 1
        irh3pg = nrat
        ratnam(irh3pg) = 'r_h1_h3_to_he4'

        nrat = nrat + 1
        irhe4gp = nrat
        ratnam(irhe4gp) = 'r_he4_to_h1_h3'

        if (icoords) then
         call coords(iprot,ih3)
         call coords(iprot,ihe4)
         call coords(ih3,ihe4)
        end if

! t(t,2n)he4
        nrat = nrat + 1
        irtt2n = nrat
        ratnam(irtt2n) = 'r_h3_h3_to_neut_neut_he4'

        nrat = nrat + 1
        irhe42nt = nrat
        ratnam(irhe42nt) = 'r_neut_neut_he4__to_h2_h3'

        if (icoords) then
         call coords(ineut,ih3)
         call coords(ineut,ihe4)
         call coords(ih3,ihe4)
        end if
       end if





! if we have he3
       if (ihe3 .ne. 0) then

! he3(he3,2p)he4
        nrat = nrat + 1
        ir33 = nrat
        ratnam(ir33) = 'r_he3_he3_to_h1_h1_he4'

        nrat = nrat + 1
        ir33inv = nrat
        ratnam(ir33inv) = 'r_h1_h1_he4_to_he3_he3'

        if (icoords) then
         call coords(iprot,ihe3)
         call coords(iprot,ihe4)
         call coords(ihe3,ihe4)
        end if


! he3(p,e+nu)he4
        nrat = nrat + 1
        irhep = nrat
        ratnam(irhep) = 'r_h1_he3_wk_he4'


! he3(n,g)he4
        nrat = nrat + 1
        irhe3ng = nrat
        ratnam(irhe3ng) = 'r_he3_ng_he4'

        nrat = nrat + 1
        irhe4gn = nrat
        ratnam(irhe4gn) = 'r_he4_gn_he3'

        if (icoords) then
         call coords(ineut,ihe3)
         call coords(ineut,ihe4)
         call coords(ineut,ihe4)
        end if
       end if



! if we have he3 and deuterium

      if (ihe3 .ne. 0 .and. ih2 .ne. 0) then

! he3(d,p)he4
        nrat = nrat + 1
        irhe3dp = nrat
        ratnam(irhe3dp) = 'r_h2_he3_to_h1_he4'

        nrat = nrat + 1
        irhe4pd = nrat
        ratnam(irhe4pd) = 'r_h1_he4_to_h2_he3'

        if (icoords) then
         call coords(iprot,ih2)
         call coords(iprot,ihe3)
         call coords(iprot,ihe4)
         call coords(ih2,ihe3)
         call coords(ih2,ihe4)
         call coords(ihe3,ihe4)
        end if

! d(d,n)he3
        nrat = nrat + 1
        irddn = nrat
        ratnam(irddn) = 'r_h2_h2_to_neut_he3'

        nrat = nrat + 1
        irhe3nd = nrat
        ratnam(irhe3nd) = 'r_neut_he3_to_h2_h2'

        if (icoords) then
         call coords(ineut,ih2)
         call coords(ineut,ihe3)
         call coords(ih2,ihe4)
        end if

      end if



! if we have he3, deuterium, tritium

      if (ihe3 .ne. 0  .and. ih2 .ne. 0  .and. ih3 .ne. 0) then

! he3(t,d)he4
        nrat = nrat + 1
        irhe3td = nrat
        ratnam(irhe3td) = 'r_he3_h3_to_d_he4'

        nrat = nrat + 1
        irhe4dt = nrat
        ratnam(irhe4dt) = 'r_h2_he4_to_h3_he3'

        if (icoords) then
         call coords(ih2,ih3)
         call coords(ih2,ihe3)
         call coords(ih2,ihe4)
         call coords(ih3,ihe3)
         call coords(ih3,ihe4)
         call coords(ihe3,ihe4)
        end if

      end if


! if we have he3 and tritium

      if (ihe3 .ne. 0  .and. ih3 .ne. 0) then

! he3(t,np)he4
        nrat = nrat + 1
        irhe3tnp = nrat
        ratnam(irhe3tnp) = 'r_he3_h3_to_neut_h1_he4'

        nrat = nrat + 1
        irhe4pnt = nrat
        ratnam(irhe4pnt) = 'r_neut_h1_he4_to_h3_he3'

        if (icoords) then
         call coords(iprot,ineut)
         call coords(iprot,ih3)
         call coords(iprot,ihe3)
         call coords(iprot,ihe4)
         call coords(ineut,ih3)
         call coords(ineut,ihe3)
         call coords(ineut,ihe4)
         call coords(ih3,ihe3)
         call coords(ih3,ihe4)
         call coords(ihe3,ihe4)
        end if

      end if



! if we have li7

       if (ili7 .ne. 0) then

! li7(p,g)be8 and li7(p,a)he4
        nrat = nrat + 1
        irli7pag = nrat
        ratnam(irli7pag) = 'r_h1_li7_to_he4_he4'

        nrat = nrat + 1
        ir2he4ga = nrat
        ratnam(ir2he4ga) = 'r_he4_he4_to_h1_li7'

        if (icoords) then
         call coords(iprot,ili7)
         call coords(iprot,ihe4)
         call coords(ili7,ihe4)
        end if

       end if

! if we have li7 and deuterium

      if (ili7 .ne. 0 .and. ih2 .ne. 0) then

! li7(d,n)2a
        nrat = nrat + 1
        irli7dn = nrat
        ratnam(irli7dn) = 'r_h2_li7_to_neut_he4_he4'

        nrat = nrat + 1
        ir2and = nrat
        ratnam(ir2and) = 'r_neut_he4_he4_to_h2_li7'

        if (icoords) then
         call coords(ineut,ih2)
         call coords(ineut,ihe4)
         call coords(ineut,ili7)
         call coords(ih2,ihe4)
         call coords(ih2,ili7)
         call coords(ihe4,ili7)
        end if

      end if

! if we have li7 and tritium

      if (ili7 .ne. 0 .and. ih3 .ne. 0) then

! li7(t,2n)2a
        nrat = nrat + 1
        irli7t2n = nrat
        ratnam(irli7t2n) = 'r_h3_li7_to_neut_neut_he4_he4'

        nrat = nrat + 1
        ir2a2nt = nrat
        ratnam(ir2a2nt) = 'r_neut_neut_he4_he4_to_h3_li7'

        if (icoords) then
         call coords(ineut,ih3)
         call coords(ineut,ihe4)
         call coords(ineut,ili7)
         call coords(ih3,ihe4)
         call coords(ih3,ili7)
         call coords(ihe4,ili7)
        end if

      end if


! if we have li7 and he3

      if (ili7 .ne. 0 .and. ihe3 .ne. 0) then

! li7(he3,np)2a
        nrat = nrat + 1
        irli7he3np = nrat
        ratnam(irli7he3np) = 'r_he3_li7_to_neut_h1_he4_he4'

        nrat = nrat + 1
        ir2apnhe3 = nrat
        ratnam(ir2apnhe3) = 'r_neut_h1_he4_he4_to_he3_li7'

        if (icoords) then
         call coords(ineut,iprot)
         call coords(ineut,ihe3)
         call coords(ineut,ihe4)
         call coords(ineut,ili7)
         call coords(iprot,ihe3)
         call coords(iprot,ihe4)
         call coords(iprot,ili7)
         call coords(ihe3,ihe4)
         call coords(ihe3,ili7)
         call coords(ihe4,ili7)
        end if

      end if


! if we have be7 and deutrium
! be7(d,p)2a
       if (ibe7 .ne. 0 .and. ih2 .ne. 0) then
        nrat = nrat + 1
        irbe7dp = nrat
        ratnam(irbe7dp) = 'r_h2_be7_to_h1_he4_he4'

        nrat = nrat + 1
        ir2apd = nrat
        ratnam(ir2apd) = 'r_h1_he4_he4_to_h2_be7'

        if (icoords) then
         call coords(iprot,ih2)
         call coords(iprot,ihe4)
         call coords(iprot,ibe7)
         call coords(ih2,ihe4)
         call coords(ih2,ibe7)
         call coords(ihe4,ibe7)
        end if

       end if


! if we have be7 and tritium
! be7(t,np)2a
       if (ibe7 .ne. 0 .and. ih3 .ne. 0) then
        nrat = nrat + 1
        irbe7tnp = nrat
        ratnam(irbe7tnp) = 'r_h3_be7_to_neut_h1_he4_he4'

        nrat = nrat + 1
        ir2apnt = nrat
        ratnam(ir2apnt) = 'r_neut_h1_he4_he4_to_h3_be7'

        if (icoords) then
         call coords(ineut,iprot)
         call coords(ineut,ih3)
         call coords(ineut,ihe4)
         call coords(ineut,ibe7)
         call coords(iprot,ih3)
         call coords(iprot,ihe4)
         call coords(iprot,ibe7)
         call coords(ih3,ihe4)
         call coords(ih3,ibe7)
         call coords(ihe4,ibe7)
        end if

       end if


! if we have be7 and he3
! be7(he3,2p)2a
       if (ibe7 .ne. 0 .and. ihe3 .ne. 0) then
        nrat = nrat + 1
        irbe7he32p = nrat
        ratnam(irbe7he32p) = 'r_he3_be7_to_h1_h1_he4_he4'

        nrat = nrat + 1
        ir2a2phe3 = nrat
        ratnam(ir2a2phe3) = 'r_h1_h1_he4_he4_to_he3_be7'

        if (icoords) then
         call coords(iprot,ihe3)
         call coords(iprot,ihe4)
         call coords(iprot,ibe7)
         call coords(ihe3,ihe4)
         call coords(ihe3,ibe7)
         call coords(ihe4,ibe7)
        end if

       end if



! if we have be9
       if (ibe9 .ne. 0) then

! a(an,g)be9
        nrat = nrat + 1
        iraan = nrat
        ratnam(iraan) = 'r_neut_he4_he4_to_be9'

        nrat = nrat + 1
        irgaan = nrat
        ratnam(irgaan) = 'r_be9_to_neut_he4_he4'

        if (icoords) then
         call coords(ineut,ihe4)
         call coords(ineut,ibe9)
         call coords(ihe4,ibe9)
        end if
       end if

! if we have be9 and deuterium
       if (ibe9 .ne. 0 .and. ih2 .ne. 0) then

! be9(p,d)be8 =>2a
        nrat = nrat + 1
        irbe9pd = nrat
        ratnam(irbe9pd) = 'r_h1_be9_to_h2_he4_he4'

        nrat = nrat + 1
        ir2adp = nrat
        ratnam(ir2adp) = 'r_h2_he4_he4_to_h1_be9'

        if (icoords) then
         call coords(iprot,ih2)
         call coords(iprot,ihe4)
         call coords(iprot,ibe9)
         call coords(ih2,ihe4)
         call coords(ih2,ibe9)
         call coords(ihe4,ibe9)
        end if
       end if


! if we have b8
       if (ib8 .ne. 0) then

! b8(p=>n)be8 =>2a reactions
        nrat = nrat + 1
        irb8ep = nrat
        ratnam(irb8ep) = 'r_b8_wk_he4_he4'

        if (icoords) then
         call coords(ib8,ihe4)
        end if
       end if


! if we have b11
       if (ib11 .ne. 0) then

! b11(p,a)be8 => 2a reactions
        nrat = nrat + 1
        irb11pa = nrat
        ratnam(irb11pa) = 'r_h1_b11_to_he4_he4_he4'

        nrat = nrat + 1
        ir3ap = nrat
        ratnam(ir3ap) = 'r_he4_he4_he4_to_h1_b11'

        if (icoords) then
         call coords(iprot,ihe4)
         call coords(iprot,ib11)
         call coords(ihe4,ib11)
        end if
       end if


! if we have c11
       if (ic11 .ne. 0) then

! c11(n,a)be8 => 2a
        nrat = nrat + 1
        irc11na = nrat
        ratnam(irc11na) = 'r_c11_neut_to_he4_he4_he4'

        nrat = nrat + 1
        ir3an = nrat
        ratnam(ir3an) = 'r_he4_he4_he4_to_c11_neut'

        if (icoords) then
         call coords(ineut,ihe4)
         call coords(ineut,ic11)
         call coords(ihe4,ic11)
        end if
       end if

       if (icoords) then
        close(unit=44)
        write(6,*) 'wrote isotope_linkages.dat'
        write(6,*)
       end if


! say how many isotopes and rates 
!       write(6,*)
!       write(6,222) 'ionbeg=',ionbeg,' ionend=',ionend,' ionmax=',ionmax,' neqs=',neqs
!222    format(1x,4(a,i5))
!       write(6,222) 'nrates=',nrat 
!       write(6,*) 'minimum size of cx array ',kmax
!       write(6,*)

! say isotope names
!        write(6,*)
!        write(6,118) (i,ionam(i),aion(i),zion(i), i=1,ionmax)
!  118   format(1x,i4,' ',a,1p2e10.2)
!        write(6,*) 'leaving init_torch'
!        read(5,*)

! say rate names
!       write(6,*) 'rate names:'
!       do i=1,nrat
!        write(6,777) i,' ',ratnam(i)
!777     format(1x,i6,a,a)
!       end do

       return
       end subroutine init_torch





      subroutine coords(j,k)
      include 'implno.dek'
      include 'network.dek'

! write the coordinates of the isotopes being linked - useful for network plots

      integer :: j,k
      write(44,10) int(nion(j)), int(zion(j)), int(nion(k)), int(zion(k)), & 
                   ionam(j),ionam(k)
10    format(1x,4i4,2a8)
      return
      end subroutine coords




      subroutine naray
      include 'implno.dek'
      include 'network.dek'

! this routine builds the nrr(7,i) and nrrneut(7,i) arrays, which specify
! the location of isotopes coupled to i by various reactions. that is,
! it build what becomes the band diagonal part of the jacobian for binary reactions.

! the first index on nrr refers to reactions of the form
!      1=ng   2=pn   3=pg   4=ap   5=an   6=ag   7=b-

! second index on nrr refers to the entrance channel isotope
! the value of nrr hold the exit chanel isotope

! for example i(a,g)k is encoded as nrr(6,i) = k

!
! nrreut if for neutrino nucleosynthesis, which is disabled for now
! the first index on nrrneut refers to reactions of the form
!      1=nu,e-,n  2=nu,e-  3=nu,e-,p   4=nu e+,n   5=nu,e+   6=nu,e+,p


! declare
      integer  ::  i,k,n,kz,kn,jz(7),jn(7)


! initialize for nrr
      jz(1) = 0
      jz(2) = 1
      jz(3) = 1
      jz(4) = 1
      jz(5) = 2
      jz(6) = 2
      jz(7) = -1

      jn(1) = 1
      jn(2) = -1
      jn(3) = 0
      jn(4) = 2
      jn(5) = 1
      jn(6) = 2
      jn(7) = 1

! build nrr
      do i=ionbeg,ionend
       do n=1,7
        nrr(n,i) = 0
        kz = int(zion(i)) + jz(n)
        kn = int(nion(i)) + jn(n)
        do k=ionbeg,ionend
         if (kz.eq.int(zion(k)) .and. kn.eq.int(nion(k))) then
          nrr(n,i)=k

! these are the n-z plane coordinates connecting isotop i with isotope k
! does not pick up the specials though
!          write(6,'(1x,0p4f8.1)') nion(i), zion(i), nion(k), zion(k)
!          write(6,'(1x,4i4)') int(nion(i)), int(zion(i)), int(nion(k)), int(zion(k))
         end if

        enddo
       enddo
      enddo


! initialize for nrrneut for neutrino reactions
!      jz(1) =  1
!      jz(2) =  1
!      jz(3) =  0
!      jz(4) = -1
!      jz(5) = -1
!      jz(6) = -2
!      jz(7) = -2
!      jn(1) = -2
!      jn(2) = -1
!      jn(3) = -1
!      jn(4) = 0
!      jn(5) = 1
!      jn(6) = 1
!      jn(7) = -2

! build nrrneut
!      do i=ionbeg,ionend
!       do n=1,7
!        nrrneut(n,i) = 0
!        kz = int(zion(i)) + jz(n)
!        kn = int(nion(i)) + jn(n)
!        do k=ionbeg,ionend
!         if (kz.eq.int(zion(k)) .and. kn.eq.int(nion(k))) nrrneut(n,i)=k
!        enddo
!       enddo
!      enddo

      return
      end subroutine naray





      subroutine zet127
      include 'implno.dek'
      include 'network.dek'
!
! this routine sets up a 136 isotope torch network
!
! set the network id and name
      idnet   = idtorch
      netname = 'torch127'
      inetin    = 0


! deuterium and tritium
      inetin         = inetin + 1
      izzz(inetin)   = 1
      inmin(inetin)  = 1
      inmax(inetin)  = 2

! helium 3
      inetin         = inetin + 1
      izzz(inetin)   = 2
      inmin(inetin)  = 1
      inmax(inetin)  = 1


! lithium 7
      inetin         = inetin + 1
      izzz(inetin)   = 3
      inmin(inetin)  = 4
      inmax(inetin)  = 4

! berylium 7-8
      inetin         = inetin + 1
      izzz(inetin)   = 4
      inmin(inetin)  = 3
      inmax(inetin)  = 3

! boron 8
      inetin         = inetin + 1
      izzz(inetin)   = 5
      inmin(inetin)  = 3
      inmax(inetin)  = 3


! carbon 12-14
      inetin         = inetin + 1
      izzz(inetin)   = 6
      inmin(inetin)  = 6
      inmax(inetin)  = 8

! nitrogen 13-15
      inetin         = inetin + 1
      izzz(inetin)   = 7
      inmin(inetin)  = 6
      inmax(inetin)  = 8

! oxygen 14-19
      inetin         = inetin + 1
      izzz(inetin)   = 8
      inmin(inetin)  = 6
      inmax(inetin)  = 11

! flourine 17-21
      inetin         = inetin + 1
      izzz(inetin)   = 9
      inmin(inetin)  = 8
      inmax(inetin)  = 12

! neon 18-24
      inetin         = inetin + 1
      izzz(inetin)   = 10
      inmin(inetin)  = 8
      inmax(inetin)  = 14

! sodium 19-24
      inetin         = inetin + 1
      izzz(inetin)   = 11
      inmin(inetin)  = 8
      inmax(inetin)  = 13

! magnesium 22-27
      inetin         = inetin + 1
      izzz(inetin)   = 12
      inmin(inetin)  = 10
      inmax(inetin)  = 15

! aluminum 25-29
      inetin         = inetin + 1
      izzz(inetin)   = 13
      inmin(inetin)  = 12
      inmax(inetin)  = 16

! silicon 27-30
      inetin        = inetin + 1
      izzz(inetin)  = 14
      inmin(inetin) = 13
      inmax(inetin) = 16

! phosphorus 29-32
      inetin        = inetin + 1
      izzz(inetin)  = 15
      inmin(inetin) = 14
      inmax(inetin) = 18

! sulfer 30-34
      inetin        = inetin + 1
      izzz(inetin)  = 16
      inmin(inetin) = 14
      inmax(inetin) = 18

! clorine 33-37
      inetin        = inetin + 1
      izzz(inetin)  = 17
      inmin(inetin) = 16
      inmax(inetin) = 20

! argon 35-39
      inetin        = inetin + 1
      izzz(inetin)  = 18
      inmin(inetin) = 17
      inmax(inetin) = 21

! potasium 37-42
      inetin        = inetin + 1
      izzz(inetin)  = 19
      inmin(inetin) = 18
      inmax(inetin) = 23

! calcium 39-44
      inetin        = inetin + 1
      izzz(inetin)  = 20
      inmin(inetin) = 19
      inmax(inetin) = 24

! scandium 42-46
      inetin        = inetin + 1
      izzz(inetin)  = 21
      inmin(inetin) = 21
      inmax(inetin) = 25

! titanium 43-49
      inetin        = inetin + 1
      izzz(inetin)  = 22
      inmin(inetin) = 21
      inmax(inetin) = 27

! vandium 46-51
      inetin        = inetin + 1
      izzz(inetin)  = 23
      inmin(inetin) = 23
      inmax(inetin) = 28

! chromium 47-52
      inetin        = inetin + 1
      izzz(inetin)  = 24
      inmin(inetin) = 23
      inmax(inetin) = 28

! manganese 49-55
      inetin        = inetin + 1
      izzz(inetin)  = 25
      inmin(inetin) = 24
      inmax(inetin) = 30

! iron 51-56
      inetin        = inetin + 1
      izzz(inetin)  = 26
      inmin(inetin) = 25
      inmax(inetin) = 30

! cobolt 53-58
      inetin        = inetin + 1
      izzz(inetin)  = 27
      inmin(inetin) = 26
      inmax(inetin) = 31

! nickel 54-60
      inetin        = inetin + 1
      izzz(inetin)  = 28
      inmin(inetin) = 26
      inmax(inetin) = 32
      return
      end

