      subroutine nse(tt,dd,yye,newguess,ipartition,icoulomb, &
                     xmass_out,xmun,xmup,iprint)
      include 'implno.dek'
      include 'const.dek'
      include 'network.dek'


! this routine puts a chosen reaction network into its nse distribution.

! input:
! tt  = temperature
! dd  = density
! yye = electron mole number
! newguess   = 0 = use the old initial guess
!            = 1 = a new initial guess is made
! ipartition = 0 = use temperature independent partition functions
!            = 1 = use temperature dependent partition functions
! icoulomb   = 0 = do not use coulomb corrections
!            = 1 = use coulomb corrections
! iprint = print flag


! output:
! xmass_out  = output nse mass fractions
! xmun        = chemical potential of neutrons
! xmup        = chemical potential of protons



! declare the pass
      integer :: newguess,ipartition,icoulomb,iprint
      real*8  :: tt,dd,yye,xmass_out(1),xmun,xmup


! communicate
      real*8 ::       temp,den,ye_want,beta,mu_c_p,yei(abignet),mu_c(abignet)
      common /nsec1/  temp,den,ye_want,beta,mu_c_p,yei,mu_c


! locals
      external           :: nsefunc
      logical            :: check
      integer            :: nfev,ntaken,i
      integer, parameter :: ntrial = 200, n = 2
      real*8             :: x(n),resid(n),amass,fac1,fac2,dum,xne,ge,a3,gi,sqrtgi
      real*8, parameter  :: tolf    = 1.0d-10,              tolx  = 1.0d-12, &
                            twopi   = 2.0d0*pi,             esqu  = qe*qe, &
                            forthpi = 4.0d0 * pi/3.0d0,     third = 1.0d0/3.0d0, &
                            fivth   = 5.0d0/3.0d0,          a1    = -0.9052d0, &
                            a2      = 0.6322d0,             a2inv = 1.0d0/a2, &
                            rt3     = 1.7320508075688772d0, half_rt3 = 0.5d0 * rt3

! for the initial guess
      real*8, parameter :: hinv = 1.0d0/h, mev2erg = ev2erg*1.0d6, &
                           mev2gr  = mev2erg/clight**2,  a56 = 56.0d0, &
                           z56 = 28.0d0, n56 = 28.0d0,  b56  = 4.8398d2


! initialize a common quantity
      yei(1:ionmax) = zion(1:ionmax)/aion(1:ionmax)


! fill the common block
      temp    = tt
      den     = dd
      ye_want = yye
      beta    = 1.0d0/(kerg * temp)


! get the partition functions
      wpart(ionbeg:ionmax) = 1.0d0
      if (ipartition .eq. 1) then
       if (which_bdat .eq. 1) then
        call get_partition_functions(tt*1.0d-9)
       else
        stop 'bad which_bdat in routine nse'
       end if
      end if


! set the coulomb corrections
! calder et al, apj 656 313 2007, eq a1

      mu_c_p = 0.0d0
      mu_c(ionbeg:ionmax) = 0.0d0
      if (icoulomb .eq. 1) then
       xne    = ye_want * avo * den
       ge     = esqu * beta * (forthpi * xne)**third
       a3     = -0.5d0*sqrt(3.0d0) - a1 / sqrt(a2)

       do i=1,ionmax
        gi      = zion(i)**(fivth)  * ge
        sqrtgi  = sqrt(gi)
        mu_c(i) = a1*(sqrt(gi*(a2+gi)) &
                   - a2*log(sqrt(gi*a2inv) + sqrt(1.0d0 + gi*a2inv)) &
                   + 2.0d0*a3*(sqrtgi - atan(sqrtgi)))
       enddo
       mu_c_p = mu_c(iprot)
      end if



! here is an initial guess for the neutron and proton chemical potentials,
! (x1) and (x2) respectively. obtained by setting xmass(ini56) = 1,
! setting mup = mun, and inverting the saha equation.
! here we use pure ni56 as it emprically appears to be a
! robust guess for all temp, rho, ye combinations.

      if (newguess .eq. 1) then
       newguess = 0
       amass  = n56*mn + z56*mp - b56*mev2gr
       fac1   = a56/(avo * den)
       fac2   = twopi/(beta*h) * amass*hinv
       fac2   = fac2 * sqrt(fac2)
       x(1)   = -(log(fac1*fac2)/beta + b56*ev2erg*1.0d6)/a56
       x(2)   = x(1)
      end if



! root find on mass and charge conservation for
! the chemical potentials of protons and neutrons

      call xnewt_nse(ntrial,x,n,tolx,tolf,ntaken,check,nfev,nsefunc)


! be sure we converged
      if (check .or. ntaken .eq. ntrial) then
       write(6,*)
       write(6,*) 'in routine nse, check convergence of xnewt_nse root finder'
       write(6,*)
      end if


! some optional diagnostics
      if (iprint .eq. 1) then
       write(6,*)
       write(6,110) 'iterations taken             =',ntaken
       write(6,110) 'function evals               =',nfev
       write(6,111) 'roots                        =',x(1),x(2)
       call nsefunc(dum,x,resid)
       write(6,111) 'mass conservation   residual =',resid(1)
       write(6,111) 'charge conservation residual =',resid(2)

 110   format(1x,a,i4)
 111   format(1x,a,1p2e14.6)
      end if


! fill the output array using the converged values

      call nsefunc(dum,x,resid)


! bound the output nse abundances
      xmass_out(1:ionmax) = min(1.0d0,max(xmass_nse(1:ionmax),1.0d-30))
      xmun = x(1)
      xmup = x(2)

      return
      end subroutine nse







      subroutine nsefunc(x,y,f)
      include 'implno.dek'
      include 'const.dek'
      include 'network.dek'


! this routine returns the root find functions.
! input is the point x and y a vector of the unknowns.
! output is the vector of root find functions f, which should be the
! zero vector upon convergence.

! y(1) is input as the neutron chemical potential
! y(2) is input as the proton chemical potential


! declare the pass
      real*8 :: x,y(*),f(*)


! locals
      integer           :: i,indx(abignet),j,ifirst
      real*8            ::  ye,mu,fac1,fac2,fac3,xmsum,sum2,deninv,ww
      real*8, parameter :: hinv    = 1.0d0/h, twopih  = 2.0d0 * pi/h, &
                           mev2erg = ev2erg*1.0d6, mev2gr  = mev2erg/clight**2

! communicate
      real*8 ::       temp,den,ye_want,beta,mu_c_p,yei(abignet),mu_c(abignet)
      common /nsec1/  temp,den,ye_want,beta,mu_c_p,yei,mu_c


! chemical potential and mass fraction of each isotope
! hartmann et al, apj 297 837 1985, eq 2
! calder et al, apj 656 313 2007, eq a1

      deninv = 1.0d0/den
      ww     = twopih/beta

      do i=1,ionmax
       mu           = nion(i)*y(1) + zion(i)*y(2)
       fac1         = mion(i) * deninv * wpart(i)
       fac2         = ww * mion(i) * hinv
       fac2         = fac2*sqrt(fac2)
       fac3         = exp( beta * (mu + bion(i)*mev2erg) - mu_c(i) + zion(i)*mu_c_p)
       xmass_nse(i) = fac1 * fac2 * fac3
      enddo


! sum the mass fractions - mass conservation
! minimize roundoff by adding in ascending order

      call indexx(ionmax,xmass_nse,izwork1) 
      xmsum = 0.0d0
      do i=1,ionmax
       xmsum = xmsum + xmass_nse(izwork1(i))
      end do
!      xmsum = sum(xmass_nse(1:ionmax))


! form ye - charge conservation
! this formulation assumes mion(i) = aion(i)*amu

      zwork1(1:ionmax) = yei(1:ionmax) * xmass_nse(1:ionmax)
      call indexx(ionmax,zwork1,izwork1) 
      ye = 0.0d0
      do i=1,ionmax
       ye = ye + zwork1(izwork1(i))
      end do
!      ye = sum(yei(1:ionmax) * xmass_nse(1:ionmax))



! this formulation does not assume mion(i) = aion(i)*amu
! seitenzahl et al 2007

      sum2 = sum(amu/mion(1:ionmax) * ((ye_want - 1.0d0)*zion(1:ionmax) &
                 + ye_want * nion(1:ionmax)) * xmass_nse(1:ionmax) )


! mass and charge conservation are the requirements
      f(1) = xmsum - 1.0d0
      f(2) = ye - ye_want

      f(2) = sum2

      return
      end subroutine nsefunc





      subroutine nsejac(x,y,f,dfdy,n,np)
      include 'implno.dek'
      include 'const.dek'
      include 'network.dek'

! this routine returns the functions and the jacobian to do the root find on
! input is x, and y(n) a vector of the unknowns. output is f(n)
! and its jacobian dfdy(np,np).

! y(1) is the neutron chemical potential
! y(2) is the proton chemical potential


! declare the pass
      integer :: n,np
      real*8  :: x,y(n),f(n),dfdy(np,np)

! locals
      integer :: indx(abignet),i,j
      real*8  :: xmbn(abignet),xmbp(abignet),fac6(abignet), &
                 mu,mubn,mubp,fac1,fac2,fac3,fac4,fac5, &
                 xmsum,xmsumbn,xmsumbp,ye,yebn,yebp,deninv,ww,sum2,sum2bn,sum2bp
      real*8, parameter :: hinv  = 1.0d0/h, twopih  = 2.0d0 * pi/h, &
                           mev2erg = ev2erg*1.0d6, mev2gr  = mev2erg/clight**2

! communicate
      real*8 ::      temp,den,ye_want,beta,mu_c_p,yei(abignet),mu_c(abignet)
      common /nsec1/ temp,den,ye_want,beta,mu_c_p,yei,mu_c



! chemical potential and mass fraction of each isotope
! hartmann et al, apj 297 837 1985, eq 2
! calder et al, apj 656 313 2007, eq a1

      deninv = 1.0d0/den
      ww     = twopih/beta

! loop over isotopes
      do i=1,ionmax
       mu       = nion(i) * y(1) + zion(i) * y(2)
       mubn     = nion(i)
       mubp     = zion(i)

       fac1     = mion(i) * deninv * wpart(i)
       fac2     = ww * mion(i) * hinv
       fac2     = fac2 * sqrt(fac2)
       fac3     = exp( beta * (mu + bion(i) * ev2erg * 1.0d6) &
                           - mu_c(i) + zion(i)*mu_c_p)
       fac4     = fac1 * fac2 * fac3

       xmass_nse(i) = fac4
       xmbn(i)      = fac4 * beta * mubn
       xmbp(i)      = fac4 * beta * mubp

      enddo


! sum the mass fractions - mass conservation
      xmsum   = sum(xmass_nse(1:ionmax))
      xmsumbn = sum(xmbn(1:ionmax))
      xmsumbp = sum(xmbp(1:ionmax))


! form ye - charge conservation 
! this formulation assumes mion(i) = aion(i)*amu
      ye   = sum(yei(1:ionmax)*xmass_nse(1:ionmax))
      yebn = sum(yei(1:ionmax)*xmbn(1:ionmax))
      yebp = sum(yei(1:ionmax)*xmbp(1:ionmax))


! this formulation does not assume mion(i) = aion(i)*amu
! seitenzahl et al 2007

     fac6(1:ionmax) = amu/mion(1:ionmax) * ((ye_want - 1.0d0)*zion(1:ionmax) + ye_want*nion(1:ionmax))
     sum2   = sum(fac6(1:ionmax) * xmass_nse(1:ionmax)) 
     sum2bn = sum(fac6(1:ionmax) * xmbn(1:ionmax)) 
     sum2bp = sum(fac6(1:ionmax) * xmbp(1:ionmax)) 


! mass and charge conservation are the requirements
      f(1) = xmsum - 1.0d0
      f(2) = ye - ye_want

      f(2) = sum2


! jacobian
      dfdy(1,1) = xmsumbn ; dfdy(1,2) = xmsumbp
      dfdy(2,1) = yebn    ; dfdy(2,2) = yebp

      dfdy(2,1) = sum2bn  ; dfdy(2,2) = sum2bp

      return
      end subroutine nsejac





      subroutine xnewt_nse(ntrial,x,n,tolx,tolf,ntaken,check,nfev,func)
      include 'implno.dek'


! given an initial guess x(1:n) for the root of n equations, this routine
! finds the root by a globally convergent newtons method. the vector of
! functions to be zeroed, called fvec(1:n) in the routine below, is
! returned by the user supplied routine func. the output quantity check
! is false on nomal return and true if the routine has converged to a
! local minimum of the function xfminx_nse. if so, try restarting from a
! different initial guess.

! np is the maximum number of equations n
! ntrial is the maximum number of iterations to try
! ntaken is the number of iterations done
! tolf sets the convergence on function values
! tolmin sets the criterion for deciding wheather spurious convergence to
!        a false minimum of xfminx_nse has occured
! tolx is the convergence criteria on deltax
! stpmx is the scaled maximum step length allowed in the line searches
! nfev is the number of function evaluations


! declare the pass
      external :: func
      logical  :: check
      integer  :: ntrial,n,ntaken,nfev
      real*8   :: x(n),tolx,tolf


! common block communication with routine xfminx_nse
      integer, parameter :: np=4
      integer            :: nn
      real*8             :: fvec(np)
      common /newtnse/      fvec,nn


! locals
      integer :: i,its,j,indx(np)
      real*8  :: d,den,f,fold,stpmax,xsum,temp,test, &
                 fjac(np,np),g(np),p(np),xold(np),xfminx_nse,dum
      real*8, parameter :: tolmin = 1.0d-12, stpmx = 2.0d0



! initialize
      if (n .gt. np) stop 'n > np in routine xnewt'
      nn   = n ; f = xfminx_nse(x,func) ; nfev = 1 ; ntaken = 0

!  test for the initial guess being a root, using a more stringent tolf
      test = maxval(abs(fvec(1:n)))
      if (test .lt. 0.01*tolf) then
       check = .false.
       return
      end if


! get stpmax for the line search
      xsum = sum(x(1:n)*x(1:n))
      stpmax = stpmx * max(sqrt(xsum),dfloat(n))


! start of iteration loop; get the jacobian
      do its = 1, ntrial
       ntaken = its


! second order accurate numerical jacobian
!       call jac_nse(dum,x,fjac,n,n,np,np,func)
!       nfev = nfev + 2*n + 1


! analytic jacobian
       call nsejac(dum,x,fvec,fjac,n,np)
       nfev = nfev + 1


! compute grad f for the line searches
       do i=1,n
        g(i) = sum(fjac(1:n,i)*fvec(1:n))
       enddo


! store x, and f and form right hand sides
       xold(1:n) = x(1:n)  ; fold = f  ;  p(1:n) = -fvec(1:n)


! solve the linear systems

!       write(6,*)
!       write(6,112) x(1),x(2)
!       write(6,112) fvec(1),fvec(2)
!       write(6,112) fjac(1,1),fjac(1,2),fjac(2,1),fjac(2,2)
! 112   format(1x,1p2e14.6)


       call ludcmp(fjac,n,np,indx,d)
       call lubksb(fjac,n,np,indx,p)


! line search returns new x and f
! it also gets fvec at the new x when it calls xfminx_nse

       call lnsrch_nse(n,xold,fold,g,p,x,f,stpmax,check,nfev,func)

!       write(6,112) x(1),x(2)
!       write(6,112) f
!       write(6,*)

! test for convergence on function value
       test = maxval(abs(fvec(1:n)))
       if (test .lt. tolf) then
        check = .false.
        return
       end if

! check for zero gradiant, i.e. spurious convergence
       if (check) then
        den  = max(f, 0.5d0 * n)
        test = maxval( abs(g(1:n) * max(abs(x(:1:n)),1.0d0)/den) )
        if (test .lt. tolmin) then
         check = .true.
        else
         check = .false.
        end if
        return
       end if

! test for convergence on deltax
       test = maxval((abs(x(1:n)-xold(1:n)))/max(abs(x(1:n)),1.0d0))
       if (test .lt. tolx) return

!       write(6,*) its,test

! back for another iteration
      enddo
      check = .true.
      return
      end subroutine xnewt_nse




      subroutine lnsrch_nse(n,xold,fold,g,p,x,f,stpmax,check,nfev,func)
      include 'implno.dek'

! given an n dimensional point xold(1:n), the value of the function fold
! and the gradient g(1:n) at the point, and a direction p(1:n), this routine
! finds a new point x(1:n) along the direction of p from xold where the
! function xfminx_nse has decreased "sufficiently". the new function value is
! returned in f. stpmax is an input quanity that limits the length of the
! steps so that the function is not evaluated in regions where it is
! undefined or subject to overflow. p is usually the newton direction. the
! output quantity check is false on normal exit, and true when x is too
! close to xold. in a minimization routine, this usually signals
! convergence and can be ignored. however, in a root finding routine, the
! calling routine should check wheather the convergence is spurious.


! declare the pass
      external :: func
      logical  :: check
      integer  :: n,nfev
      real*8   :: f,fold,stpmax,g(n),p(n),x(n),xold(n)


! locals
      integer :: i
      real*8  :: xfminx_nse,a,alam,alam2,alamin,b,disc,f2,rhs1, &
                 rhs2,slope,xsum,temp,test,tmplam

! alf ensures decreases in the function value, tolx is the convergence criterion on deltax
      real*8, parameter :: alf  = 1.0d-4, tolx = 3.0d-12, alam_start = 1.0d0


! initialize and scale if the attempted step is too big
      check = .false.
      xsum = sqrt(sum(p(1:n)*p(1:n)))
      if (xsum .gt. stpmax) then
       xsum = 1.0d0/xsum
       p(1:n) = p(1:n) * stpmax*xsum
      end if
      slope = sum(g(1:n)*p(1:n))
      if (slope .ge. 0.0) stop 'roundoff problem in lnsrch_nse'

! compute lambda_min
      test = maxval(abs(p(1:n))/max(abs(xold(1:n)),1.0d0))
      alamin = tolx/test


! always try a full newton step, start of iteration loop
      alam = alam_start
1     continue
      do i=1,n
       x(i) = xold(i) + alam*p(i)


! for the nse problem make sure the neutron and proton
! chemical potentials are less than or equal to zero.
! hmm for low ye (0.2) and high density (1e14), the neutron
! chemical potential does indeed go positive. let's try
! letting it go wherever it wants

!       if (x(i) .gt. 0.0) x(i) = xold(i) + 1.0d-4*alam*p(i)
       if (x(i) .gt. 0.0) x(i) = 1.0d-10
      enddo

      f    = xfminx_nse(x,func)
      nfev = nfev + 1


! convergence on deltax, for root finding, the calling routine
! should verify the convergence
      if (alam .lt. alamin) then
       x(1:n) = xold(1:n)
       check = .true.
       return

! sufficient function decrease
      else if (f .le. fold + alf*alam*slope) then
       return

! backtrack
      else
       if (alam .eq. alam_start) then
        tmplam = -slope / (2.0d0 * (f-fold-slope))
       else
        rhs1 = f  - fold - alam*slope
        rhs2 = f2 - fold - alam2*slope
        a    = (rhs1/alam**2 - rhs2/alam2**2)/(alam-alam2)
        b    = (-alam2*rhs1/alam**2 + alam*rhs2/alam2**2) / (alam-alam2)
        if (a .eq. 0.0) then
         tmplam = -slope/(2.0d0 * b)
        else
         disc = b*b - 3.0d0 * a * slope
         if (disc .lt. 0.0) then
          tmplam = 0.5d0 * alam
         else if (b .le. 0.0) then
          tmplam = (-b + sqrt(disc)) / (3.0d0 * a)
         else
          tmplam = -slope/(b + sqrt(disc))
         end if
        end if
        if (tmplam .gt. 0.5d0*alam) tmplam = 0.5d0*alam
       end if
      end if

! store for the next trip through
      alam2 = alam
      f2    = f
      alam  = max(tmplam, 0.1d0*alam)
      goto 1
      end subroutine lnsrch_nse






      real*8 function xfminx_nse(x,func)
      include 'implno.dek'

! returns f = 0.5 f dot f at x. func is a user supplied routine of the
! functions to be root found.
!
! common block communicates values back to routine xnewt

! declare the pass
      external   :: func
      real*8     :: x(1)


! common block communication with routine xfminx_nse
      integer, parameter :: np=4
      integer            :: nn
      real*8             :: fvec(np)
      common /newtnse/      fvec,nn

! locals
      integer  ::  i
      real*8   :: xsum,dum
!
      call func(dum,x,fvec)
      xsum = sum(fvec(1:nn)*fvec(1:nn))
      xfminx_nse = 0.5d0 * xsum
      return
      end function xfminx_nse




      subroutine jac_nse(x,y,dfdy,mcol,nrow,mmax,nmax,derivs)
      include 'implno.dek'

! this routine computes a second order accurate jacobian matrix
! of the function contained in the routine derivs.
!
! input is the point x and the the vector y at which to compute the jacobian dfdy.
!
! y has logical dimension nrow and physical dimension nmax,
! dfdy has logical dimension (mcol,nrow) and physical dimension (mmax,nmax)
!
! uses 2*nrow + 1 function evaluations


! declare the pass
      external     :: derivs
      integer      :: mcol,nrow,mmax,nmax
      real*8       :: x,y(nmax),dfdy(mmax,nmax)


! locals
      integer            :: i,j
      integer, parameter :: imax = 4
      real*8             :: fminus(imax),fplus(imax),temp,h,hinv
      real*8, parameter  :: rel = 3.162278d-8, ax = 1.0e-16

! check
       if (nrow .gt. imax) stop 'nrow > imax in jac_nse'


! for each row, get the right stepsize
      do j=1,nrow
       temp = y(j)
       h    = rel * max(abs(y(j)),ax)
       y(j) = temp + h
       h    = y(j) - temp
       call derivs(x,y,fplus)
       y(j) = temp

       temp = y(j)
       y(j) = temp - h
       h    = temp - y(j)
       call derivs(x,y,fminus)
       y(j) = temp

! compute the jth row of the jacobian
        hinv = 1.0d0/(2.0d0 * h)
        do i=1,mcol
         dfdy(i,j) = (fplus(i) - fminus(i)) * hinv
        enddo
       enddo

! restore the original state
      call derivs(x,y,fplus)
      return
      end subroutine jac_nse
